#include "StdAfx.h"
#include <sstream>
#include ".\imap4server.h"
#include "general_strings.h"
#include "mleak.h"

const static char IMAP4_RES_SEARCH[]			= "* SEARCH";
const static char IMAP4_FETCH_BODY_KEY[]		= "BODY[TEXT]";
const static char IMAP4_FETCH_HEADER_KEY[]		= "BODY[HEADER] {";

const static char ERR_CLOSE_FAILD[]				= "G[iCLOSEsj";
const static char ERR_INVALID_ACC_PASS[]		= "AJEg܂̓pX[hႢ܂";
const static char ERR_EXAMINE_FAILD[]			= "G[iEXAMINEsj";
const static char ERR_SELECT_FAILD[]			= "G[iSELECTsj";
const static char ERR_UNEXPECTED_TAG_ORDER[]	= "G[izO^Oj";
const static char ERR_CANT_MARK_DEL[]			= "[폜ŃG[܂";
const static char ERR_GET_MAIL_LIST[]			= "[Xg擾ɃG[܂";
const static char ERR_GET_MAIL[]				= "[擾ɃG[܂";
const static char ERR_GET_MAIL_HEADERS[]		= "[wb_擾ɃG[܂[0]";
const static char ERR_UID_NOT_FOUND[]			= "[wb_擾ɃG[܂[1]";
const static char ERR_UNKNOWN_UID[]				= "[wb_擾ɃG[܂[2]";
const static char ERR_CONNECTION_READ_FAIL[]	= "ʐMG[iRecvsj";
const static char ERR_COMMAND_STLS[]			= "STARTTLSR}hɎs܂";

const char CImap4Server::RES_OK[] = "OK";
const LPCSTR CImap4Server::AUTH_TYPE_NAME[AUTH_NUM] =
{
	"Text"
};

const static char LOG_HEADER_CLIENT[]		= "C";


//******************************************************************
// 
//******************************************************************

CImap4Server::CImap4Server(ServerInfo &info, CComLog *pcl, HANDLE hStop)
: CMailServer(info, pcl, hStop)
{
}

CImap4Server::~CImap4Server(void)
{
}

//******************************************************************
// 
//******************************************************************

u_short CImap4Server::GetDefaultPort(RE_SECURITY security)
{
	return (security == RS_SSL) ? IMAP4S_PORT: IMAP4_PORT;
}

void CImap4Server::GetUIDList(MailIDMap &map)
{
	_ASSERT( IsConnected() );
	std::string line;
	std::string uid;
	unsigned long no;

	SelectInboxWrite();

	// R}hM
	LogSendString("get UID SEARCH ALL\r\n");

	// X|XM
	do
	{
		LogRecvLine(line);
		if( line.compare(0, LEN_OF(IMAP4_RES_SEARCH), IMAP4_RES_SEARCH) == 0 )
		{
			LPCSTR st, ed;

			st = line.c_str() + LEN_OF(IMAP4_RES_SEARCH);
			for(;;)
			{
				while( *st == ' ' ) st++;

				no = strtoul(st, (char**)&ed, 10);
				if( no == 0 )
				{
					throw CRiffException(ERR_GET_MAIL_LIST);
				}

				uid.assign(st, ed - st);
				map[uid] = no;
				if( *ed != ' ' ) break;
				st = ed;
			}
		}
	}
	while( line[0] == '*' );
}

void CImap4Server::GetMail(LPCSTR uid, std::string &mail, int linenum)
{
	_ASSERT( IsConnected() );
	char buf[CMD_BUF_LEN];
	std::string line;

//	SelectInboxWrite();
	SelectInboxRead();

	// wb_M
	{
		// R}hM
		_snprintf_s(buf, sizeof(buf),
			"get UID FETCH %s (BODY[HEADER])\r\n",
			uid );
		LogSendString(buf);


		// X|XM
		LogRecvLine(line);
		while( line[0] == '*' )
		{
			LPCSTR pos;

			// bZ[W̊Jnʒu
			pos = strstr(line.c_str(), IMAP4_FETCH_HEADER_KEY);
			if( pos != NULL )
			{
				int len = atoi(pos + LEN_OF(IMAP4_FETCH_HEADER_KEY));
				DynamicArray<char> buf(len);

				// bZ[Wobt@Ɏ荞
				RecvLiteralString(buf, len);

				mail.assign(buf, len);
				LogRecvLine(line);	// ǂݎ̂
			}
			LogRecvLine(line);
		}

		if( ! IsGoodResponce(line.c_str(), "get") )
			throw CRiffException(ERR_GET_MAIL);
	}

	// {fBM
	if( linenum != 0 )
	{
		// R}hM
		_snprintf_s(buf, sizeof(buf),
			(linenum > 0) ?
				"get UID FETCH %s (BODY[TEXT]<0.%d>)\r\n":
				"get UID FETCH %s (BODY[TEXT])\r\n",
			uid, linenum * 100 );
		LogSendString(buf);


		// X|XM
		LogRecvLine(line);
		while( line[0] == '*' )
		{
			LPCSTR pos;

			// bZ[W̊Jnʒu
			pos = strstr(line.c_str(), IMAP4_FETCH_BODY_KEY);
			if( pos != NULL )
			{
				pos = strstr(pos, " {");
				if( pos != NULL ){
					int len = atoi(pos + 2);
					DynamicArray<char> buf(len);

					// bZ[Wobt@Ɏ荞
					RecvLiteralString(buf, len);

					mail.append(buf, len);
					LogRecvLine(line);	// ǂݎ̂
				}
			}
			LogRecvLine(line);
		}

		if( ! IsGoodResponce(line.c_str(), "get") )
			throw CRiffException(ERR_GET_MAIL);
	}
}

void CImap4Server::GetMailHeaders(NumberMap &mails)
{
	_ASSERT( IsConnected() );
	std::string line;
	std::ostringstream oss;
	NumberMap::const_iterator i;

	if( mails.size() == 0 ) return;

	SelectInboxRead();

	// R}hM
	i = mails.begin();
	oss << "get UID FETCH " << i->second->uid();
	for( ++i; i != mails.end(); ++i )
	{
		oss << ',' << i->second->uid();
	}
	oss << " (BODY[HEADER] UID)\r\n";
	LogSendString(oss.str().c_str());

	// X|XM
	LogRecvLine(line);
	while( line[0] == '*' )
	{
		LPCSTR pos;

		// bZ[W̊JnʒuT
		pos = strstr(line.c_str(), IMAP4_FETCH_HEADER_KEY);
		if( pos != NULL )
		{
			int len = atoi(pos + LEN_OF(IMAP4_FETCH_HEADER_KEY));
			DynamicArray<char> buf(len + 1);

			// bZ[Wobt@Ɏ荞
			RecvLiteralString(buf, len);

			// UID 擾
			LogRecvLine(line);
			pos = strstr(line.c_str(), "UID ");
			if( pos == NULL )
				throw CRiffException(ERR_UID_NOT_FOUND);

			int uid = atoi(pos + 4);
			NumberMap::iterator i = mails.find(uid);
			if( i == mails.end() )
				throw CRiffException(ERR_UNKNOWN_UID);
			i->second->SetFromHeader(buf);
		}
		LogRecvLine(line);
	}

	if( ! IsGoodResponce(line.c_str(), "get") )
		throw CRiffException(ERR_GET_MAIL_HEADERS);
}

void CImap4Server::DeleteMail(LPCSTR uid)
{
	_ASSERT( IsConnected() );
	char buf[CMD_BUF_LEN];

	SelectInboxWrite();

	_snprintf_s(buf, sizeof(buf),
		"del UID STORE %s +FLAGS (\\Deleted)\r\n",
		uid );
	LogSendString(buf);
	if( ! CheckRes("del") ) throw CRiffException(ERR_CANT_MARK_DEL);

	CloseMailbox();
}

void CImap4Server::Authorize()
{
	_ASSERT( IsConnected() );
	char buf[CMD_BUF_LEN];

	// STLS
	if( m_info.security == RS_STLS )
	{
		LogSendString("stls STARTTLS\r\n");
		if( ! CheckRes("stls") ) throw CRiffException(ERR_COMMAND_STLS);
		m_socket.SSLConnect();
	}

	// LOGIN
	_snprintf_s(buf, sizeof(buf),
		"in LOGIN %s xxxxxxxx\r\n",
		m_info.user.c_str() );
	DoLog(LOG_HEADER_CLIENT, buf);

	_snprintf_s(buf, sizeof(buf),
		"in LOGIN %s %s\r\n",
		m_info.user.c_str(), m_info.pass.c_str() );
	m_socket.SendString(buf);

	if( ! CheckRes("in") ) throw CRiffException(ERR_INVALID_ACC_PASS);
}

void CImap4Server::Quit()
{
	_ASSERT( IsConnected() );

	try{
		LogSendString("exit LOGOUT\r\n");
	}catch(...){
	}
}

//******************************************************************
// 
//******************************************************************

void CImap4Server::SelectInboxRead()
{
	LogSendString("selr EXAMINE INBOX\r\n");
	if( ! CheckRes("selr") ) throw CRiffException(ERR_EXAMINE_FAILD);
}

void CImap4Server::SelectInboxWrite()
{
	LogSendString("selw SELECT INBOX\r\n");
	if( ! CheckRes("selw") ) throw CRiffException(ERR_SELECT_FAILD);
}

void CImap4Server::CloseMailbox()
{
	LogSendString("close CLOSE\r\n");
	if( ! CheckRes("close") ) throw CRiffException(ERR_CLOSE_FAILD);
}

bool CImap4Server::IsGoodResponce(LPCSTR line, LPCSTR tag)
{
	int taglen = strlen(tag);

	if( strncmp(tag, line, taglen) != 0 )
		throw CRiffException(ERR_UNEXPECTED_TAG_ORDER);

	return strncmp(line + taglen + 1, RES_OK, RES_OK_LEN) == 0;
}

// X|X`FbN
bool CImap4Server::CheckRes(LPCSTR tag)
{
	std::string line;

	LogRecvLine(line);
	while( line[0] == '*' )
	{
		LogRecvLine(line);
	}

	if( line[0] == '+' ) return false;
	return IsGoodResponce(line.c_str(), tag);
}

void CImap4Server::RecvLiteralString(LPSTR buf, int len)
{
	int read = 0;
	LPSTR pos;

	pos = buf;
	while( len > 0 )
	{
		read = m_socket.Recv(pos, len);
		if( read == 0 )
			throw CRiffException(ERR_CONNECTION_READ_FAIL);

		pos += read;
		len -= read;
	}
}
