#include "StdAfx.h"
#include <shellapi.h>
#include ".\mailcheckerthread.h"
#include "codec.h"
#include "RiffSettings.h"
#include "general_strings.h"
#include "mleak.h"

const static LPCSTR UI_UPDATING				= "\XV";
const static LPCSTR CONNECTING				= "T[o֐ڑ";
const static LPCSTR DISCONNECTING			= "ؒf";
const static LPCSTR GETTING_MAIL			= "[̎擾";
const static LPCSTR GETTING_MAIL_LIST		= "[ꗗ擾";
const static LPCSTR MAIL_CONVERTING			= "[ϊ";
const static LPCSTR EXTRACT_DIFFS			= "𒊏o";
const static LPCSTR GETTING_HEADER_FORMAT	= "%d̃wb_擾";
const static LPCSTR AUTO_PREVIEW			= "vr[";

const static char DELETTING_MAIL[]			= "[폜";

const static char ERR_DIS_AT_CONNECT[]		= "T[oɐڑɐؒf܂";
const static char ERR_DIS_AT_GET_LIST[]		= "[ꗗ̎擾ɐؒf܂";
const static char ERR_DIS_AT_GET_HEADER[]	= "wb_̎擾ɐؒf܂";
const static char ERR_DIS_AT_DISCONNECT[]	= "T[oؒfOɐؒf܂";
const static char ERR_MAIL_NOT_EXISTS[]		= "w肳ꂽ[݂܂";

//******************************************************************
// 
//******************************************************************

CMailCheckerThread::CMailCheckerThread(CRiffNsm *pNsm, CGID gid, AccSettings &set, int nAcc, HANDLE hStop)
: m_nsm(pNsm, gid, set.u_set)
, m_timerID(NULL)
, m_mm(gid, set.u_set.account.c_str())
, m_set(set.c_set)
, m_pServer(CreateServer(set.c_set.protocol, set.info, pNsm, hStop))
, m_nAcc(nAcc)
, m_bCheckMail(false)
, m_bKeepConnection(false)
{
}

CMailCheckerThread::~CMailCheckerThread(void)
{
	delete m_pServer;
}

//******************************************************************
// 
//******************************************************************

void CMailCheckerThread::DoBeginCheck(void)
{
	if( m_bCheckMail ) return;

	if( m_bKeepConnection )
	{
		try{
			m_nsm.NotifyState(CONNECTING);
			m_pServer->Connect();
		}catch(CRiffException e){
			if( m_pServer->IsConnected() )
				m_pServer->Disconnect();

			if( e.GetErrorMsg() != NULL ){
				m_nsm.Error(e);
				m_nsm.NotifyDefaultState();
			}
		}
	}

	m_bCheckMail = true;
	m_nsm.BeginCheck();
}

void CMailCheckerThread::DoEndCheck(void)
{
	if( ! m_bCheckMail ) return;

	if( m_pServer->IsConnected() )
		m_pServer->Disconnect();

	m_bCheckMail = false;
	m_nsm.EndCheck();
}

void CMailCheckerThread::DoCheckNow(void)
{
	NumberMap add;
	GIDList del;

	// XV擾
	if( ! GetMailDiffs(add, del) )
		return;

	// 폜ꂽ[[Xg폜
	for( GIDList::const_iterator igid = del.begin(); igid != del.end(); ++igid ){
		m_mm.Delete(*igid);
	}

	// V[[Xgɒǉ
	m_mm.Add(add);

	// UIXV
	m_nsm.NotifyState(UI_UPDATING);
	UIMailSync();
	m_nsm.NewMail(add);

	if( (add.size() > 0) || (del.size() > 0) )
		m_mm.SaveMails();

	m_nsm.NotifyDefaultState();
}

void CMailCheckerThread::DoConfigChanged(int nAcc)
{
	if( nAcc != m_nAcc && nAcc != -1 ) return;
	AccSettings set;

	if( m_nAcc >= 0 ){
		RIFF_SETTINGS.GetAccSettings(m_nAcc, set);
		// TODO: ڑ̍XVKv
		m_pServer->ConfigChanged(set.info);
	}else{
		RIFF_SETTINGS.GetDefaultSettings(set);
	}

	m_set = set.c_set;
	m_nsm.ConfigChanged(set.u_set);
	m_nsm.MailSync(m_mm.GetNumberMap());

	if( m_timerID != NULL ){
		EndTimer();
		BeginTimer();
	}
}

void CMailCheckerThread::DoOpenSetting(HNsmConnection hConnection)
{
	if( m_nAcc >= 0 ){
		RIFF_SETTINGS.ShowAccOptionDialog(hConnection, m_nAcc);
	}
}

void CMailCheckerThread::DoDeleteMail(CGID gid, HWND hParent)
{
	const CMailInfo *pmi = m_mm[gid];

	if( pmi == NULL ){
		m_nsm.Error(CRiffException(ERR_MAIL_NOT_EXISTS));
		return;
	}
	if( ! m_nsm.ConfirmDelete(*pmi, hParent) ) return;

	try{
		if( ! m_bKeepConnection )
		{
			m_nsm.NotifyState(CONNECTING);
			m_pServer->Connect();
		}

		m_nsm.NotifyState(DELETTING_MAIL);
		m_pServer->DeleteMail(pmi->uid());

		if( ! m_bKeepConnection )
		{
			m_nsm.NotifyState(DISCONNECTING);
			m_pServer->Disconnect();
		}
	}catch(CRiffException e){
		if( ! m_bKeepConnection )
		{
			if( m_pServer->IsConnected() )
				m_pServer->Disconnect();
		}

		if( e.GetErrorMsg() == NULL ) return;

		m_nsm.Error(e);
		m_nsm.NotifyDefaultState();
		return;
	}
	m_nsm.NotifyDefaultState();

	m_mm.Delete(gid);
	m_mm.SaveMails();
	m_nsm.MailSync(m_mm.GetNumberMap());
}

void CMailCheckerThread::DoViewMail(CGID gid)
{
	std::string mail;
	CMailInfo *pmi = m_mm[gid];

	if( pmi == NULL ) return;

	try{
		if( ! m_bKeepConnection )
		{
			m_nsm.NotifyState(CONNECTING);
			m_pServer->Connect();
		}

		m_nsm.NotifyState(GETTING_MAIL);
		m_pServer->GetMail(pmi->uid(), mail, m_set.preview_linenum);

		if( ! m_bKeepConnection )
		{
			m_nsm.NotifyState(DISCONNECTING);
			m_pServer->Disconnect();
		}

		m_nsm.NotifyState(MAIL_CONVERTING);
		m_nsm.ViewMail(*pmi, mail);
		if( ! pmi->SetPreviewed() )
			m_mm.SaveMails();
	}catch(CRiffException e){
		if( ! m_bKeepConnection )
		{
			if( m_pServer->IsConnected() )
				m_pServer->Disconnect();
		}

		if( e.GetErrorMsg() == NULL ) return;

		m_nsm.Error(e);
	}

	m_nsm.NotifyDefaultState();
}

void CMailCheckerThread::DoRunMUA()
{
	LPCSTR path = RIFF_SETTINGS.GetMuaPath();
	std::string wd = path;
	int pos;

	pos = wd.find_last_of('\\');
	if( pos > 0 ){
		wd.resize(pos);
		::ShellExecute(NULL, NULL, path, m_set.mua_option.c_str(), wd.c_str(), SW_SHOWNORMAL);
	}
}

//******************************************************************
// 
//******************************************************************

void CMailCheckerThread::Init()
{
	m_nsm.NotifyState(UI_UPDATING);
	m_nsm.MailSync(m_mm.GetNumberMap());
	m_nsm.NotifyDefaultState();
}

void CMailCheckerThread::BeginTimer()
{
	if( m_timerID != NULL ) return;

	m_timerID = ::SetTimer(NULL, NULL, m_set.interval * 1000, NULL);
}

void CMailCheckerThread::EndTimer()
{
	if( m_timerID == NULL ) return;

	::KillTimer(NULL, m_timerID);
	m_timerID = NULL;
}

void CMailCheckerThread::UIMailSync()
{
	m_nsm.MailSync(m_mm.GetNumberMap());
}

bool CMailCheckerThread::GetMailDiffs(NumberMap &add, GIDList &del)
{
//	int count = 0;
//	int getnum;
	MailIDMap list;
	NumberMap::reverse_iterator mail;
	char buf[64];
	std::string header;
	LPCSTR disconnect_msg = NULL_STRING;

	try{
		if( ! m_bKeepConnection )
		{
			disconnect_msg = ERR_DIS_AT_CONNECT;
			m_nsm.NotifyState(CONNECTING);
			m_pServer->Connect();
		}

		disconnect_msg = ERR_DIS_AT_GET_LIST;
		m_nsm.NotifyState(GETTING_MAIL_LIST);
		m_pServer->GetUIDList(list);

		m_nsm.NotifyState(EXTRACT_DIFFS);
		m_mm.ExtractMailDiff(list, add, del);

/*		// V[̏擾
		getnum = add.size();
		for( mail = add.rbegin(); mail != add.rend(); ++mail )
		{
			sprintf(buf, GETTING_HEADER_FORMAT, ++count, getnum);
			m_nsm.NotifyState(buf);

			m_pServer->GetMailHeader(mail->first, header);
			mail->second->SetFromHeader(header.c_str());
		}
*/
		// V[̏񓯊擾
		disconnect_msg = ERR_DIS_AT_GET_HEADER;
		sprintf_s(buf, GETTING_HEADER_FORMAT, add.size());
		m_nsm.NotifyState(buf);
		m_pServer->GetMailHeaders(add);

		// auto preview
		if( m_set.auto_preview && ((int)add.size() <= m_set.max_auto_preview) )
		{
			std::string mail_text;

			m_nsm.NotifyState(AUTO_PREVIEW);

			for( mail = add.rbegin(); mail != add.rend(); ++mail )
			{
				CMailInfo &mi = *(mail->second);

				m_pServer->GetMail(mi.uid(), mail_text, m_set.preview_linenum);
				m_nsm.ViewMail(mi, mail_text);
				mi.SetPreviewed();
			}
		}

		if( ! m_bKeepConnection )
		{
			disconnect_msg = ERR_DIS_AT_DISCONNECT;
			m_pServer->Disconnect();
		}
	}catch(CRiffException e){
		if( ! m_bKeepConnection )
		{
			if( m_pServer->IsConnected() )
				m_pServer->Disconnect();
		}

		for( mail = add.rbegin(); mail != add.rend(); ++mail )
			delete mail->second;

		if( e.GetErrorMsg() != NULL ){
			if( e.GetErrorMsg() == ERR_DISCONNECTED )
				e.ReplaceErrorMsg(disconnect_msg);
			m_nsm.Error(e);
			m_nsm.NotifyDefaultState();
		}
		return false;
	}

	return true;
}

//******************************************************************
// 
//******************************************************************

typedef VOID (CALLBACK *TimerProc)(HWND, UINT, UINT_PTR, DWORD);

void CMailCheckerThread::RiffThreadBody()
{
	MSG msg;

	Init();

	BeginTimer();
	while ( ::GetMessage(&msg, NULL, 0, 0) > 0 )
    {
		switch(msg.message){
		case WM_TIMER:
			if( (m_timerID != NULL) && (msg.wParam == m_timerID) ){
				m_bCheckMail ? DoCheckNow(): UIMailSync();
				while(::PeekMessage(&msg, (HWND)-1, WM_TIMER, WM_TIMER, PM_REMOVE)){
					if( msg.message == WM_QUIT ) return;
				}
			}
			break;
		case WM_USER_VIEW_MAIL:		DoViewMail(msg.lParam); break;
		case WM_USER_RUN_MUA:		DoRunMUA(); break;
		case WM_USER_CHECK_NOW:		DoCheckNow(); break;
		case WM_USER_DELETE_MAIL:	DoDeleteMail(msg.lParam, (HWND)msg.wParam); break;
		case WM_USER_CONFIG_UPDATED:DoConfigChanged(msg.lParam); break;
		case WM_USER_OPEN_SETTING:	DoOpenSetting((HNsmConnection)msg.lParam); break;
		case WM_USER_BEGIN_CHECK:
			DoBeginCheck();
			DoCheckNow();
			break;
		case WM_USER_END_CHECK:		DoEndCheck(); break;
		case WM_USER_EXIT:			EndTimer(); return;
		default:					break;
		}
	}
	EndTimer();
}
