#include "StdAfx.h"
#include <strstream>
#include ".\mailinfo.h"
#include "mail.h"
#include "codec.h"
#include "general_strings.h"
#include "mleak.h"

//******************************************************************
// 
//******************************************************************

CMailInfo::CMailInfo(CGID gid)
: m_gid(gid)
, m_recv_time(0)
{
	Init();
}

CMailInfo::~CMailInfo(void)
{
}

//******************************************************************
// 
//******************************************************************

void CMailInfo::SetFromHeader(LPCSTR header)
{
	std::string enc;

	ExtractHeaderField(header, "From:", enc);
	Decode(enc, m_from);

	ExtractHeaderField(header, "To:", enc);
	Decode(enc, m_to);

	ExtractHeaderField(header, "Subject:", enc);
	Decode(enc, m_subject);

	ExtractHeaderField(header, "Date:", enc);
	SetDateFromStr(enc.c_str());
}

#define DEF_SIZE 32
LPCSTR CMailInfo::dateStr() const
{
	static char buf[DEF_SIZE];
	tm date_tm;

	if( m_date < 0 ) return "(MȂ)";

	localtime_s(&date_tm, &m_date);
	strftime(buf, DEF_SIZE, "%Y/%m/%d %H:%M:%S ", &date_tm);
	sprintf_s(buf+strlen(buf), 6, "%+05d", m_tz);
	return buf;
}

void CMailInfo::operator <<(std::ifstream &f)
{
	CBase64Filter filter(f);

	f >> m_recv_time;
	f >> m_uid;
	filter >> m_subject;
	filter >> m_from;
	filter >> m_to;
	f >> m_date;
	f >> m_tz;
	f >> m_previewed;
}

void CMailInfo::operator >>(std::ofstream &f) const
{
	CBase64Filter filter(f);

	f << m_recv_time << SPACE_DELIMITER;
	f << m_uid << SPACE_DELIMITER;
	filter << m_subject << SPACE_DELIMITER;
	filter << m_from << SPACE_DELIMITER;
	filter << m_to << SPACE_DELIMITER;
	f << m_date << SPACE_DELIMITER;
	f << m_tz << SPACE_DELIMITER;
	f << (m_previewed ? 1: 0) << SPACE_DELIMITER;
	f << std::endl;
}

//******************************************************************
// 
//******************************************************************

// Fri, 28 Nov 2003 00:03:26 +0900
/*
#define WEEKDAY		0
#define DAY			5
#define MONTH		8
#define YEAR		12
#define HOUR		17
#define MINUTE		20
#define SECOND		23
#define TIMEZONE	26

#define C2I(a, b, c) (((a)<<16) | ((b)<<8) | (c))
#define JAN	C2I('J','a','n')
#define FEB	C2I('F','e','b')
#define MAR	C2I('M','a','r')
#define APR	C2I('A','p','r')
#define MAY	C2I('M','a','y')
#define JUN	C2I('J','u','n')
#define JUL	C2I('J','u','l')
#define AUG	C2I('A','u','g')
#define SEP	C2I('S','e','p')
#define OCT	C2I('O','c','t')
#define NOV	C2I('N','o','v')
#define DEC	C2I('D','e','c')
*/
void CMailInfo::SetDateFromStr(LPCSTR str)
{
	static const char m[12][4] = {
		"jan","feb","mar","apr","may","jun","jul","aug","sep","oct","nov","dec"
	};
	struct tm t;

	t.tm_mday = NextNumber(str);
	while( *str && ! isalpha(*str) ) str++;
	for( int i = 0; i < 12; i++ ){
		if( _strnicmp(str, m[i], 3) == 0 ){
			t.tm_mon = i;
			break;
		}
	}
	t.tm_year = NextNumber(str) - 1900;
	t.tm_hour = NextNumber(str);
	t.tm_min = NextNumber(str);
	t.tm_sec = NextNumber(str);
	m_tz = NextNumber(str);

	t.tm_isdst = 0;
	m_date = mktime(&t);
}

int CMailInfo::NextNumber(LPCSTR &str)
{
	int num;
	while( *str && ! is_atoi_chr(*str) ) str++;
	num = atoi(str);
	while( is_atoi_chr(*str) ) str++;
	return num;
}

