#include "StdAfx.h"
#include <process.h>
#include ".\mailserver.h"
#include "general_strings.h"
#include "mleak.h"

const static char ERR_SERVER_NOT_FOUND[]	= "T[o܂";
const static char ERR_TIMEOUT_RESOLVE[]		= "T[ỏɃ^CAEg܂";

const static char LOG_HEADER_CLIENT[]		= "C";
const static char LOG_HEADER_SERVER[]		= "S";

//******************************************************************
// 
//******************************************************************

CMailServer::CMailServer(ServerInfo &info, CComLog *pcl, HANDLE hStop)
: m_info(info)
, m_pLog(pcl)
, m_socket(hStop, info.timeout, info.security)
, m_hStop(hStop)
{
}

CMailServer::~CMailServer(void)
{
}

//******************************************************************
// public
//******************************************************************

// ڑ
void CMailServer::Connect()
{
	_ASSERT( ! IsConnected() );
	ConnectToHost();
	Authorize();
}

// ؒf
void CMailServer::Disconnect()
{
	_ASSERT( IsConnected() );
	Quit();
	m_socket.Disconnect();
	DoLog("-", "Disconnected");
}

//******************************************************************
// protected
//******************************************************************

// kI[񑗐MOo
void CMailServer::LogSendString(LPCSTR str)
{
	DoLog(LOG_HEADER_CLIENT, str);
	m_socket.SendString(str);
}

// MkI[Oo
void CMailServer::LogRecvLine(std::string &line)
{
	m_socket.RecvLine(line);
	DoLog(LOG_HEADER_SERVER, line.c_str());
}

//******************************************************************
// private
//******************************************************************

// zXg֐ڑ
void CMailServer::ConnectToHost()
{
	struct sockaddr_in addr;
	struct in_addr host;
	u_short port;

	// ڑ擾
	if( (m_info.host.length() > 0) && isdigit(m_info.host[m_info.host.length()-1]) ){
		host.S_un.S_addr = inet_addr(m_info.host.c_str());
	}else{
		host = GetHostByName(m_info.host.c_str());
		if( host.S_un.S_addr == NULL )
			throw CRiffException(ERR_SERVER_NOT_FOUND);
	}
	port = m_info.port ? m_info.port: GetDefaultPort(m_info.security);

	// ʐMO
	{
		char buf[40];
		_snprintf_s(buf, sizeof(buf), "Connecting to %d.%d.%d.%d:%d",
			host.S_un.S_un_b.s_b1, host.S_un.S_un_b.s_b2,
			host.S_un.S_un_b.s_b3, host.S_un.S_un_b.s_b4,
			port
		);
		DoLog("-", buf);
	}

	// ڑ
	addr.sin_family = PF_INET;
	addr.sin_addr = host;
	addr.sin_port = htons(port);
	m_socket.Connect(addr);
}

// 荞݉\zXg
struct in_addr CMailServer::GetHostByName(LPCSTR name)
{
	enum{
		EVENT_STOP,
		EVENT_THREAD,
		NUM_OF_EVENTS
	};
	HANDLE evts[NUM_OF_EVENTS];
	int ret;
	struct in_addr value;

	evts[EVENT_STOP] = m_hStop;
	evts[EVENT_THREAD] = (HANDLE)_beginthreadex(NULL, 0, AsyncGetHostByName, (LPVOID)name, 0, NULL);
	ret = WaitForMultipleObjects(NUM_OF_EVENTS, evts, FALSE, m_info.timeout);
	if( ret == WAIT_TIMEOUT ) throw CRiffException(ERR_TIMEOUT_RESOLVE);

	GetExitCodeThread(evts[EVENT_THREAD], (LPDWORD)&value);
	CloseHandle(evts[EVENT_THREAD]);

	if( ret == (WAIT_OBJECT_0 + EVENT_STOP) )
		throw CRiffException(NULL);

	return value;
}

// zXgXbh
unsigned int __stdcall CMailServer::AsyncGetHostByName(void* param)
{
	struct hostent *pHost = gethostbyname((LPCSTR)param);
	if( (pHost != NULL) && (pHost->h_length > 0) )
		return *(unsigned int*)pHost->h_addr;
	return NULL;
}

