#pragma once
#include <string>
#include "ComLog.h"
#include "MailServerSocket.h"
#include "setting_struct.h"
#include "mail_info.h"

//*****************************************************

//*****************************************************

class CMailServer
{
private:
	CComLog * const m_pLog;
	HANDLE m_hStop;

protected:
	CMailServerSocket m_socket;
	ServerInfo m_info;

public:
	CMailServer(ServerInfo &info, CComLog *pcl, HANDLE hStop);
	const CMailServer& operator =(const CMailServer&)
	{ _ASSERT(FALSE); }
	virtual ~CMailServer(void);

public:
	void Connect();
	void Disconnect();

public:
	virtual void GetUIDList(MailIDMap &map) = 0;
	virtual void GetMail(LPCSTR uid, std::string &mail, int linenum) = 0;
	virtual void GetMailHeaders(NumberMap &mails) = 0;
	virtual void DeleteMail(LPCSTR uid) = 0;
	virtual u_short GetDefaultPort(RE_SECURITY security) = 0;

public:
	void ConfigChanged(ServerInfo &info)
	{
		const bool bSecurityUpdated = (info.security != m_info.security);
		m_info = info;
		m_socket.SetTimeout(info.timeout);
		if( bSecurityUpdated )
		{
			if( IsConnected() )
			{
				Disconnect();
			}
			m_socket.SetSecurity(info.security);
		}
	}
//	void GetMailHeader(int mail_no, std::string &header)
//	{ GetMail(mail_no, header, 0); }
	bool IsConnected() const
	{ return m_socket.IsConnected(); }

protected:
	virtual void Authorize() = 0;
	virtual void Quit() = 0;

protected:
	void LogSendString(LPCSTR str);
	void LogRecvLine(std::string &line);

	void DoLog(LPCSTR header, LPCSTR body)
	{
		if( m_info.bDoLog )
			m_pLog->DoLog(header, body);
	}

private:
	void ConnectToHost();
	struct in_addr GetHostByName(LPCSTR name);
	static unsigned int __stdcall AsyncGetHostByName(void* param);
};
