#include "StdAfx.h"
#include <process.h>
#include <algorithm>
#include ".\mailserversocket.h"
#include "general_strings.h"
#include <openssl/crypto.h>
#include <openssl/rand.h>

#include "mleak.h"

const static char ERR_CANT_CREATE_SOCKET[]	= "\Pbg̍쐬Ɏs܂";
const static char ERR_CANT_CONNECT_SERVER[]	= "T[oɐڑł܂";
const static char ERR_CANT_SEND[]			= "MɎs܂";
const static char ERR_TIMEOUT[]				= "MɃ^CAEg܂";

const static char ERR_WAIT_FAILED[]			= "Cxg҂Ɏs܂";
const static char ERR_ENUM_EVENTS[]			= "Cxg񋓂Ɏs܂";
const static char ERR_EVENT_SELECT[]		= "CxgZbgɎs܂";

const static char ERR_SSL[]					= "SSL̃G[ł";

const static char LOG_HEADER_CLIENT[]		= "C";
const static char LOG_HEADER_SERVER[]		= "S";


//******************************************************************
// public
//******************************************************************

CMailServerSocket::CMailServerSocket(HANDLE hStop, int timeout, RE_SECURITY security)
: m_timeout(timeout)
, m_buf_filled(0)
, m_socket(INVALID_SOCKET)
, m_security(security)
, m_ssl(NULL)
, m_ssl_ctx(NULL)
{
	m_hEvents[EVENT_STOP] = hStop;
	m_hEvents[EVENT_SOCKET] = WSACreateEvent();
	_ASSERT( m_hEvents[EVENT_SOCKET] != WSA_INVALID_EVENT );
}

CMailServerSocket::~CMailServerSocket(void)
{
	WSACloseEvent(m_hEvents[EVENT_SOCKET]);
}

//******************************************************************
// public
//******************************************************************

// zXg֐ڑ
void CMailServerSocket::Connect(sockaddr_in &saddr)
{
	_ASSERT( ! IsConnected() );
	SOCKET s;
	int ret;

	// \Pbg쐬
	s = socket(PF_INET, SOCK_STREAM, 0);
	if( s == INVALID_SOCKET ){
		throw CRiffException(ERR_CANT_CREATE_SOCKET);
	}
	ret = FALSE;
//	ret = setsockopt(s, IPPROTO_TCP, TCP_NODELAY, (const char*)&ret, sizeof(ret));
//	if( ret == SOCKET_ERROR ) throw ERR_SET_NODELAY;
	ret = WSAEventSelect(s, m_hEvents[EVENT_SOCKET], FD_READ|FD_WRITE|FD_CONNECT|FD_CLOSE);
	if( ret == SOCKET_ERROR ) throw CRiffException(ERR_EVENT_SELECT);

	// ڑ
	ret = connect(s, (sockaddr*)&saddr, sizeof(saddr));
	if( (ret != 0) && (WSAGetLastError() != WSAEWOULDBLOCK) ){
		closesocket(s);
		throw CRiffException(ERR_CANT_CONNECT_SERVER);
	}
	m_socket = s;

	ret = WaitAsyncSocket(FD_CONNECT_BIT);
	if( ret != 0 ){
		closesocket(s);
		m_socket = INVALID_SOCKET;
		throw CRiffException(ERR_CANT_CONNECT_SERVER);
	}

	if( m_security == RS_SSL )
	{
		try
		{
			SSLConnect();
		}
		catch(...)
		{
			SSLDisconnect();
			throw;
		}
	}
}

// ؒf
void CMailServerSocket::Disconnect()
{
	_ASSERT( IsConnected() );
	if( IsSSL() )
	{
		SSLDisconnect();
	}
	else
	{
		char buf[256];

		shutdown(m_socket, SD_BOTH);
		while( Recv(buf, sizeof(buf)) > 0 );
		closesocket(m_socket);
		m_socket = INVALID_SOCKET;
	}
}

void CMailServerSocket::SSLConnect()
{
	int ret;

	m_ssl_ctx = SSL_CTX_new(SSLv23_client_method());
	if( m_ssl_ctx == NULL )	throw CRiffException(ERR_SSL);

	m_ssl = SSL_new(m_ssl_ctx);
	if( m_ssl == NULL )	throw CRiffException(ERR_SSL);

	ret = SSL_set_fd(m_ssl, m_socket);
	if( ret == 0 )	throw CRiffException(ERR_SSL);

	for(;;)
	{
		ret = SSL_connect(m_ssl);

		switch( ret )
		{
		case 0:	throw CRiffException(ERR_SSL);	// fail but cleanly shutdown
		case 1:	return;							// success
		default:	// fail and not cleanly shutdown or non-blocking continue
			SSLWaitAsyncSocket(ret);
			break;
		}
	}
}

void CMailServerSocket::SSLDisconnect()
{
	if( m_ssl != NULL )		SSLShutdown();
	closesocket(m_socket);
	if( m_ssl != NULL )		SSL_free(m_ssl);
	if( m_ssl_ctx != NULL ) SSL_CTX_free(m_ssl_ctx);
	m_socket	= INVALID_SOCKET;
	m_ssl		= NULL;
	m_ssl_ctx	= NULL;
}

void CMailServerSocket::SSLShutdown()
{
	try
	{
		int sd = SSL_get_shutdown(m_ssl);
		
		// close notify 𑗂ĂȂΑ
		if( (sd & SSL_SENT_SHUTDOWN) == 0 )
		{
			for(;;)
			{
				int ret = SSL_shutdown(m_ssl);
				if( ret >= 0 ) break;
				SSLWaitAsyncSocket(ret);
			}
		}
	}
	catch(...)
	{
		// ignore exception
	}
}

void CMailServerSocket::SSLWaitAsyncSocket(int ret)
{
	int reason = SSL_get_error(m_ssl, ret);

	switch( reason )
	{
	case SSL_ERROR_WANT_READ:	WaitAsyncSocket(FD_READ_BIT);	break;
	case SSL_ERROR_WANT_WRITE:	WaitAsyncSocket(FD_WRITE_BIT);	break;
	default: throw CRiffException(ERR_SSL);
	}
}

// kI[񑗐M
void CMailServerSocket::SendString(LPCSTR str)
{
	int len = strlen(str);
	Send(str, len);
}

// 1sM
void CMailServerSocket::RecvLine(std::string &line)
{
	char buf[1024];
	int rcv_len;

	line = "";
	do{
		rcv_len = Recv(buf, sizeof(buf), '\n');
		if( rcv_len <= 0 ) throw CRiffException(ERR_DISCONNECTED);
		line.append(buf, rcv_len);
	}while( buf[rcv_len-1] != '\n' );
}

// M
void CMailServerSocket::Send(LPCSTR buf, int len)
{
	int ret;
	int send_len = 0;
	
	while( send_len < len ){
		if( IsSSL() )
		{
			ret = SSL_write(m_ssl, buf, len);

			if( ret <= 0 )
			{
				SSLWaitAsyncSocket(ret);
			}
			else
			{
				send_len += ret;
			}
		}
		else
		{
			ret = send(m_socket, buf, len, 0);

			if( ret <= 0 )
			{
				if( WSAGetLastError() != WSAEWOULDBLOCK ) return;
				WaitAsyncSocket(FD_WRITE_BIT);
			}
			else
			{
				send_len += ret;
			}
		}

//_RPT2(_CRT_WARN, "Send(%d) => ret = %d\n", len, ret);
		if( ret == SOCKET_ERROR ){
			if( WSAGetLastError() != WSAEWOULDBLOCK )
				return;
//				throw ERR_CANT_SEND;

			WaitAsyncSocket(FD_WRITE_BIT);
		}else{
			send_len += ret;
		}
	}
}

// M
int CMailServerSocket::Recv(LPSTR buf, int len)
{
	int ret;

	if( m_buf_filled <= 0 )
	{
		ret = RecvInternal(buf, len);
	}
	else
	{
		ret = ReadFromBuffer(buf, len);
	}

	return ret;
}

// c܂ŎM
int CMailServerSocket::Recv(LPSTR buf, int len, char c)
{
	int readsize = 0;
	LPCSTR pos;
	int datalen;
	int readable_max;

	if( m_buf_filled > 0 ){
		pos = (LPCSTR)memchr(m_buf, c, m_buf_filled);
		datalen = min(len, ((pos != NULL) ? pos - m_buf + 1: m_buf_filled));
		readsize = ReadFromBuffer(buf, datalen);
		if( pos != NULL ) return datalen;
	}

	_ASSERT( (m_buf_filled == 0) || (readsize == 0) );

	while( readsize < len ){
		readable_max = min(INTERNAL_BUFFER_SIZE, len - readsize);
		datalen = RecvInternal(buf + readsize, readable_max);
		if( datalen == 0 ) throw "d";
		pos = (LPCSTR)memchr(buf + readsize, c, datalen);
		if( pos != NULL )
		{
			pos++;
			SaveToBuffer(pos, buf + readsize + datalen - pos);
			return pos - buf;
		}
		readsize += datalen;
	}
	return readsize;
}

//******************************************************************
// private
//******************************************************************

int CMailServerSocket::RecvInternal(void *buf, int len)
{
	int ret;

	for(;;){
		if( IsSSL() )
		{
			ret = SSL_read(m_ssl, reinterpret_cast<char*>(buf), len);

			if( ret >= 0 )
			{
				return ret;
			}
			else
			{
				SSLWaitAsyncSocket(ret);
			}
		}
		else
		{
			ret = recv(m_socket, reinterpret_cast<char*>(buf), len, 0);

			if( ret >= 0 )
			{
				return ret;
			}
			else
			{
				if( WSAGetLastError() != WSAEWOULDBLOCK ) return 0;
				WaitAsyncSocket(FD_READ_BIT);
			}
		}
	}
}

int CMailServerSocket::ReadFromBuffer(void *buf, int len)
{
	int ret = min(m_buf_filled, len);

	memcpy(buf, m_buf, ret);
	m_buf_filled -= ret;
	memmove(m_buf, m_buf + ret, m_buf_filled);

	return ret;
}

void CMailServerSocket::SaveToBuffer(const void *buf, int len)
{
	_ASSERT( len <= INTERNAL_BUFFER_SIZE - m_buf_filled );

	memcpy(m_buf + m_buf_filled, buf, len);
	m_buf_filled += len;
}

// 񓯊҂
int CMailServerSocket::WaitAsyncSocket(int evt)
{
	int ret;
	WSANETWORKEVENTS ne;

	for(;;){
		ret = WSAWaitForMultipleEvents(NUM_OF_EVENTS, m_hEvents, FALSE, m_timeout, FALSE);
//_RPT2(_CRT_WARN, "Wait(%d) => ret = %d\n", evt, ret);
		// waitAv`FbN
		if( ret == WSA_WAIT_TIMEOUT )
			throw CRiffException(ERR_TIMEOUT);
		if( ret == WSA_WAIT_FAILED )
			throw CRiffException(ERR_WAIT_FAILED);
		if( ret == (WSA_WAIT_EVENT_0 + EVENT_STOP) )
			throw CRiffException(NULL);

		// socket Av`FbN
		ZeroMemory(&ne, sizeof(ne));
		ret = WSAEnumNetworkEvents(m_socket, m_hEvents[EVENT_SOCKET], &ne);
//_RPT3(_CRT_WARN, "Enum(%d) => ret = %d, ne = %06X\n", evt, ret, ne.lNetworkEvents);
		if( ret == SOCKET_ERROR )
			throw CRiffException(ERR_ENUM_EVENTS);
		if( (ne.lNetworkEvents >> evt) & 0x1 )
			return ne.iErrorCode[evt];
		if( ne.lNetworkEvents & FD_CLOSE )
			throw CRiffException(ERR_DISCONNECTED);
	}
}
