#include "StdAfx.h"
#include <process.h>
#include ".\pop3server.h"
#include <openssl/md5.h>
#include "general_strings.h"
#include "mleak.h"

const static LPCSTR RES_OK		= "+OK";
const static int RES_OK_LEN		= 3;
const static LPCSTR RES_ERR		= "-ERR";
const static int RES_ERR_LEN	= 4;

const char CPop3Server::AUTH_TYPE_NAME[][5] = {
	"",
	"TEXT",
	"APOP",
};

const static char ERR_COMMAND_UIDL[]		= "UIDLR}hT|[gĂ܂";
const static char ERR_COMMAND_DEL[]			= "[폜ŃG[܂";
const static char ERR_COMMAND_APOP[]		= "APOPT|[gĂ܂";
const static char ERR_COMMAND_STAT[]		= "STATR}hs܂";
const static char ERR_COMMAND_STLS[]		= "STLSR}hs܂";
const static char ERR_GET_MAIL[]			= "[擾ŃG[܂";
const static char ERR_NOT_POP3[]			= "POP3T[oł͂܂";
const static char ERR_INVALID_ACCOUNT[]		= "AJEgႢ܂";
const static char ERR_INVALID_ACC_PASS[]	= "AJEg܂̓pX[hႢ܂";
const static char ERR_CANT_SEND[]			= "MɎs܂";
const static char ERR_MAIL_NOT_EXISTS[]		= "w肳ꂽ[݂܂";

const static char LOG_HEADER_CLIENT[]		= "C";

//******************************************************************
// 
//******************************************************************

CPop3Server::CPop3Server(ServerInfo &info, CComLog *pcl, HANDLE hStop)
: CMailServer(info, pcl, hStop)
{
}

CPop3Server::~CPop3Server(void)
{
}

//******************************************************************
// public
//******************************************************************

// UIDXg擾
void CPop3Server::GetUIDList(MailIDMap &map)
{
	_ASSERT( IsConnected() );

	GetUIDMap(map);
}

// [擾
void CPop3Server::GetMail(LPCSTR uid, std::string &mail, int linenum)
{
	_ASSERT( IsConnected() );
	char buf[CMD_MAX_LEN];
	std::string line;
	int mail_no = UIDtoMailNo(uid);

	mail = "";

	// RETR or TOP M
	if( linenum < 0 ){
		_snprintf_s(buf, sizeof(buf), "RETR %d\r\n", mail_no);
	}else{
		_snprintf_s(buf, sizeof(buf), "TOP %d %d\r\n", mail_no, linenum);
	}
	LogSendString(buf);

	// X|XM
	LogRecvLine(line);
	if( ! IsGoodResponce(line.c_str()) )
		throw CRiffException(ERR_GET_MAIL);

	// [M
	for(;;){
		m_socket.RecvLine(line);
		if( line == ".\r\n" ) break;

		if( line.substr(0, 2) == ".." )
		{
			mail += line.c_str() + 1;
		}
		else
		{
			mail += line;
		}
	}
}

// [폜
void CPop3Server::DeleteMail(LPCSTR uid)
{
	_ASSERT( IsConnected() );
	char buf[CMD_MAX_LEN];
	int mail_no = UIDtoMailNo(uid);

	// DELE M
	_snprintf_s(buf, sizeof(buf), "DELE %d\r\n", mail_no);
	LogSendString(buf);

	if( ! CheckRes() )
		throw CRiffException(ERR_COMMAND_DEL);
}

void CPop3Server::GetMailHeaders(NumberMap &mails)
{
	_ASSERT( IsConnected() );
	char buf[CMD_MAX_LEN];
	HANDLE hThread;
	NumberMap::const_iterator mail;
	ThreadParam p;
	LPCSTR msg;

	if( mails.size() == 0 ) return;

	// MXbhN
	p.pThis = this;
	p.mails = &mails;
	hThread = (HANDLE)_beginthreadex(NULL, 0, AsyncGetMailHeader, &p, 0, NULL);

	// TOP M
	for( mail = mails.begin(); mail != mails.end(); ++mail )
	{
		_snprintf_s(buf, sizeof(buf), "TOP %d 0\r\n", mail->first);
		LogSendString(buf);
	}

	// MXbhI҂
	WaitForSingleObject(hThread, INFINITE);

	GetExitCodeThread(hThread, (LPDWORD)&msg);
	CloseHandle(hThread);

	if( msg != NULL ) throw CRiffException(msg);
}

//******************************************************************
// protected / override
//******************************************************************

// F
void CPop3Server::Authorize()
{
	_ASSERT( IsConnected() );
	std::string line;

	// greetingM
	LogRecvLine(line);
	if( ! IsGoodResponce(line.c_str()) ) throw CRiffException(ERR_NOT_POP3);

	// STLS
	if( m_info.security == RS_STLS )
	{
		LogSendString("STLS\r\n");
		if( ! CheckRes() ) throw CRiffException(ERR_COMMAND_STLS);
		m_socket.SSLConnect();
	}

	// Fؕ@ŕ
	switch( m_info.auth ){
	case TEXT: AuthText(); break;
	case APOP: AuthApop(line.c_str()); break;
	default: AuthAuto(line.c_str()); break;
	}
}

// ʐMI
void CPop3Server::Quit()
{
	_ASSERT( IsConnected() );
	try{
		LogSendString("QUIT\r\n");
		CheckRes();
	}catch(...){
	}
}

//******************************************************************
// private
//******************************************************************

int CPop3Server::UIDtoMailNo(LPCSTR uid)
{
	MailIDMap map;
	MailIDMap::iterator id;

	GetUIDMap(map);
	id = map.find(uid);
	if( id == map.end() ){
		throw CRiffException(ERR_MAIL_NOT_EXISTS);
	}
	return id->second;
}

void CPop3Server::GetUIDMap(MailIDMap &map)
{
	LPCSTR st, ed;
	int no;
	std::string uid;
	std::string line;

	// UIDL M
	LogSendString("UIDL\r\n");
	if( ! CheckRes() ) throw CRiffException(ERR_COMMAND_UIDL);

	// UIDXgM
	for(;;){
		// M
		m_socket.RecvLine(line);
		if( line == ".\r\n" ) break;

		// ؂o
		no = atoi(line.c_str());
		st = strchr(line.c_str(), ' ');
		if( st == NULL ) throw CRiffException(ERR_COMMAND_UIDL);
		st++;
		for( ed = st; (*ed > ' ') && (*ed != '\0'); ed++);
		uid.assign(st, ed - st);

		// UIDZbg
		map[uid] = no;
	}
}

// 탌X|Xǂ
bool CPop3Server::IsGoodResponce(LPCSTR msg)
{
	return strncmp(msg, RES_OK, RES_OK_LEN) == 0;
}

// X|X`FbN
bool CPop3Server::CheckRes()
{
	std::string line;

	LogRecvLine(line);
	return IsGoodResponce(line.c_str());
}

// Fؕ@uTEXTv
void CPop3Server::AuthText()
{
	char buf[CMD_MAX_LEN];
	int len;

	// USER/PASS M
	_snprintf_s(buf, sizeof(buf), "USER %s\r\nPASS xxxxxxxx\r\n",
		m_info.user.c_str());
	DoLog(LOG_HEADER_CLIENT, buf);

	len = _snprintf_s(buf, sizeof(buf), "USER %s\r\nPASS %s\r\n",
		m_info.user.c_str(), m_info.pass.c_str());
	m_socket.Send(buf, len);

	if( ! CheckRes() ) throw CRiffException(ERR_INVALID_ACCOUNT);
	if( ! CheckRes() ) throw CRiffException(ERR_INVALID_ACC_PASS);
}

// Fؕ@uAPOPv
void CPop3Server::AuthApop(LPCSTR greeting)
{
	LPCSTR st, ed;
	MD5_CTX md5ctx;
	std::string pass;
	unsigned char md5b[16];
	char md5a[32+1];
	char buf[CMD_MAX_LEN];

	// ڑID擾
	st = strchr(greeting, '<');
	if( st == NULL ) throw CRiffException(ERR_COMMAND_APOP);

	ed = strchr(st, '>');
	if( ed == NULL ) throw CRiffException(ERR_COMMAND_APOP);

	pass.assign(st, ed - st + 1);
	pass += m_info.pass;

	// MD5
	MD5_Init(&md5ctx);
	MD5_Update(&md5ctx, (const unsigned char*)pass.c_str(), pass.length());
	MD5_Final(md5b, &md5ctx);

	for( int i = 0; i < 16; i++ ){
		sprintf_s(&md5a[i*2], 3, "%02x", md5b[i]);
	}
	md5a[32] = '\0';

	// APOP M
	_snprintf_s(buf, sizeof(buf), "APOP %s %s\r\n", m_info.user.c_str(), md5a);
	LogSendString(buf);
	if( ! CheckRes() )
		throw CRiffException(ERR_INVALID_ACC_PASS);
}

// Fؕ@uv
void CPop3Server::AuthAuto(LPCSTR greeting)
{
	try{
		AuthApop(greeting);
	}catch(...){
		AuthText();
	}
}
/*
void CPop3Server::AuthAuth()
{
}
*/

// [擾
int CPop3Server::GetNumOfMail()
{
	std::string line;

	// STAT M
	LogSendString("STAT\r\n");
	LogRecvLine(line);

	if( ! IsGoodResponce(line.c_str()) ) 
		throw CRiffException(ERR_COMMAND_STAT);

	return atoi(line.c_str() + RES_OK_LEN + 1);
}

//******************************************************************
// thread
//******************************************************************

unsigned CPop3Server::AsyncThreadBody(const NumberMap &mails)
{
	NumberMap::const_iterator i;
	std::string mail;
	std::string line;

	try
	{
		for( i = mails.begin(); i != mails.end(); ++i ){
			// X|XM
			LogRecvLine(line);
			if( ! IsGoodResponce(line.c_str()) ){
				continue;
			}

			// [M
			mail = "";
			for(;;){
				m_socket.RecvLine(line);
				if( line == ".\r\n" ) break;
				mail += line;
			}

			i->second->SetFromHeader(mail.c_str());
		}
	}
	catch(CRiffException e)
	{
//		return (unsigned int)msg;
// TODO
	}

	return NULL;
}

unsigned __stdcall CPop3Server::AsyncGetMailHeader(void *param)
{
	ThreadParam &p = *(ThreadParam*)param;

	return p.pThis->AsyncThreadBody(*(p.mails));
}


