#include "StdAfx.h"
#include <sstream>
#include <string>
#include <shellapi.h>
#include <locale.h>
#include <uxtheme.h>
#include ".\previewdlg.h"
#include "RiffSettings.h"
#include "codec.h"
#include "mail.h"
#include "RiffNsm.h"
#include "util.h"
#include "general_strings.h"
#include "DynamicArray.h"
#include "cs_conv.h"
#include "service_name.h"

#include "mleak.h"

#define BIN_DATA_MAX_SIZE	2048
#define END_OF_MPART	(-2)
#define NO_NEXT_PART	(-1)

const static char HEADER_CONTENT_TYPE[]					= "content-type:";
const static char HEADER_CONTENT_DISPOSTION[]			= "content-disposition:";
const static char HEADER_CONTENT_TRANSFER_ENCODING[]	= "content-transfer-encoding:";

const static char TYPE_MULTIPART[]				= "multipart/";
const static char TYPE_TEXT[]					= "text/";
const static char FIELD_BOUNDARY[]				= "boundary=";
const static char FIELD_CHARSET_JIS[]			= "iso-2022-jp";
const static char FIELD_CHARSET_UTF7[]			= "utf-7";
const static char FIELD_CHARSET_UTF8[]			= "utf-8";
const static char FIELD_FILENAME_PREFIX[]		= "filename";
const static char FIELD_FILENAME2231_PREFIX[]	= "filename*";
const static char FIELD_NAME_PREFIX[]			= "name";

const static char BOUNDARY_PREFIX[]		= "\r\n--";
const static char BOUNDARY_POSTFIX[]	= "--";

const static char ENC_BASE64[]	= "base64";
const static char ENC_QUOTED[]	= "quoted-printable";

const static char ISO_2022_JP[]	= "iso-2022-jp";

const static char URL_HTTP[]	= "http://";
const static char URL_HTTPS[]	= "https://";
const static char URL_FTP[]	= 	"ftp://";
const static char PATH_REMOTE[] = "\\\\";

const static char CMB_PREFIX_HEADER[]	= "wb_";
const static char CMB_PREFIX_SINGLE[]	= "VOp[gF ";
const static char CMB_PREFIX_MULTI[]	= "}`p[g";
const static char CMB_DELIMITER[]		= "F ";

const static char EDIT_CTL_CLASS[]	= "EDIT";

WNDPROC CPreviewDlg::m_pOriginEdit = NULL;

//******************************************************************
// 
//******************************************************************

CPreviewDlg::CPreviewDlg(void)
: CDlg(IDD_PREVIEW, IDR_PREV_ACC)
{
	RIFF_SETTINGS.LoadPrevWndInfo(m_rect, m_lf, m_bForceWrap);
}

CPreviewDlg::~CPreviewDlg(void)
{
	RIFF_SETTINGS.SavePrevWndInfo(m_rect, m_bForceWrap);
}

//******************************************************************
// I[o[Ch
//******************************************************************

BOOL CPreviewDlg::DlgProc(UINT uMsg, WPARAM wParam, LPARAM lParam)
{
	switch(uMsg){
	case WM_SIZE:
		OnSize(wParam, LOWORD(lParam), HIWORD(lParam));
		break;
	case WM_MOVE:
		OnMove(LOWORD(lParam), HIWORD(lParam));
		break;
	case WM_COMMAND:
		switch( LOWORD(wParam) ){

		// p[gIR{{bNX
		case IDC_PART:
			if( HIWORD(wParam) == CBN_SELCHANGE )
				ChangePart();
			break;

		// usv{^
		case IDC_WRAP:
			ToggleWrap();
			break;

		// u[v{^
		case IDC_RUN_MUA:
			RunMUA();
			break;

		// u폜v{^
		case IDC_DEL:
			DeleteMail();
			break;

		// Ctrl+A ꂽ
		case ID_CTRL_A:
			::SendMessage(m_hBody, EM_SETSEL, 0, -1);
			break;

		default:
			return FALSE;
		}
		break;
	case WM_USER_UPDATE_FONT:
		FontUpdated();
		break;
	default:
		return FALSE;
	}
	return TRUE;
}

BOOL CPreviewDlg::InitDlgCtrl()
{
	int iBody;

	// EChȄ
	::SetWindowText(m_hDlgWnd, m_subject.c_str());
	::MoveWindow(m_hDlgWnd, m_rect.left, m_rect.top,
		m_rect.right - m_rect.left, m_rect.bottom - m_rect.top, FALSE);

	// Rg[̏
	SetItemText(IDC_SUBJECT, m_subject.c_str());
	SetItemText(IDC_FROM, m_from.c_str());
	SetItemText(IDC_DATE, m_date.c_str());
	SetCheck(IDC_WRAP, m_bForceWrap);

	// [{̏
	InitPart();

	iBody = (m_parts.size() <= 1) ? 0: 1;
	::SendMessage(GetDlgItem(IDC_PART), CB_SETCURSEL, iBody, 0);
	m_hFont = CreateFont();
	m_hBody = CreateEditBox(m_parts[iBody].c_str(), m_bForceWrap);
	::SetFocus(m_hBody);
	return FALSE;
}

BOOL CPreviewDlg::GetDlgCtrlValue()
{
	::DestroyWindow(m_hBody);
	::DeleteObject(m_hFont);
	return TRUE;
}

//******************************************************************
// Cxgnh
//******************************************************************

// EChETCYύXRg[TCYύXEChEzuL
void CPreviewDlg::OnSize(int nType, int /*width*/, int /*height*/)
{
	switch( nType ){
	case SIZE_MAXIMIZED:
	case SIZE_RESTORED:
		RECT r;
		RECT wrapR;
		RECT runR;
		RECT delR;

		::GetClientRect(m_hDlgWnd, &r);
		::GetWindowRect(GetDlgItem(IDC_WRAP), &wrapR);
		::GetWindowRect(GetDlgItem(IDC_RUN_MUA), &runR);
		::GetWindowRect(GetDlgItem(IDC_DEL), &delR);

		SetWindowPos(GetDlgItem(IDC_SUBJECT), NULL, 0, 0,
			r.right - MAIL_INFO_LEFT, MAIL_INFO_HEIGHT,
			SWP_NOMOVE|SWP_NOZORDER|SWP_NOOWNERZORDER);
		SetWindowPos(GetDlgItem(IDC_FROM), NULL, 0, 0,
			r.right - MAIL_INFO_LEFT-10, MAIL_INFO_HEIGHT,
			SWP_NOMOVE|SWP_NOZORDER|SWP_NOOWNERZORDER);
		SetWindowPos(GetDlgItem(IDC_PART), NULL,
			MAIL_BODY_HMARGIN, WRAP_BTN_TOP,
			r.right - (delR.right - wrapR.left) - MAIL_BODY_HMARGIN*2, MAIL_INFO_HEIGHT,
			SWP_NOZORDER|SWP_NOOWNERZORDER);
//		SetWindowPos(GetDlgItem(IDC_DATE), NULL, 0, 0,
//			r.right - MAIL_INFO_LEFT, MAIL_INFO_HEIGHT,
//			SWP_NOMOVE|SWP_NOZORDER|SWP_NOOWNERZORDER);

		SetWindowPos(GetDlgItem(IDC_WRAP), NULL,
			r.right - (delR.right - wrapR.left), WRAP_BTN_TOP,
			0, 0, SWP_NOSIZE|SWP_NOZORDER|SWP_NOOWNERZORDER);
		SetWindowPos(GetDlgItem(IDC_RUN_MUA), NULL,
			r.right - (delR.right - runR.left), WRAP_BTN_TOP,
			0, 0, SWP_NOSIZE|SWP_NOZORDER|SWP_NOOWNERZORDER);
		SetWindowPos(GetDlgItem(IDC_DEL), NULL,
			r.right - (delR.right - delR.left), WRAP_BTN_TOP,
			0, 0, SWP_NOSIZE|SWP_NOZORDER|SWP_NOOWNERZORDER);

		SetWindowPos(m_hBody, NULL, 0, 0,
			r.right-MAIL_BODY_HMARGIN*2, r.bottom - MAIL_BODY_TOP, SWP_NOMOVE|SWP_NOZORDER);

		UpdateWindowRect();
		break;
	}
}

// EChEړEChEzuL
void CPreviewDlg::OnMove(int /*x*/, int /*y*/)
{
	// ŏƂ͔zuLȂ
	if( ! IsIconic(m_hDlgWnd) )
		UpdateWindowRect();
}

// s؂ւ
void CPreviewDlg::ToggleWrap()
{
	std::string body;
	HWND hBody;
	int line_num, cur_line;

	m_bForceWrap = GetCheck(IDC_WRAP);

	// XN[ʒu擾
	line_num = ::SendMessage(m_hBody, EM_GETLINECOUNT, 0, 0);
	cur_line = ::SendMessage(m_hBody, EM_GETFIRSTVISIBLELINE, 0, 0);

	// GfBbg{bNX蒼
	GetItemText(m_hBody, body);
	hBody = CreateEditBox(body.c_str(), m_bForceWrap);
	::DestroyWindow(m_hBody);
	m_hBody = hBody;

	// XN[ʒu
	cur_line *= ::SendMessage(m_hBody, EM_GETLINECOUNT, 0, 0);
	::SendMessage(m_hBody, EM_LINESCROLL, 0, (cur_line/line_num));
}

// \p[g̕ύX
void CPreviewDlg::ChangePart()
{
	int iSelected = ::SendMessage(GetDlgItem(IDC_PART), CB_GETCURSEL, 0, 0);
	if( 0 <= iSelected && iSelected < (int)m_parts.size() )
		SetWindowText(m_hBody, m_parts[iSelected].c_str());
}

// {GfBbg{bNX쐬
HWND CPreviewDlg::CreateEditBox(LPCSTR text, bool bForceWrap) const
{
	HWND hEdit;
	RECT r;
	DWORD ws = ES_WANTRETURN|ES_AUTOVSCROLL|ES_MULTILINE|WS_BORDER|WS_CHILD|WS_TABSTOP|WS_CHILD|WS_VSCROLL;

	// 쐬
	if( ! bForceWrap ) ws |= WS_HSCROLL|ES_AUTOHSCROLL;
	::GetClientRect(m_hDlgWnd, &r);
	hEdit = ::CreateWindowA(EDIT_CTL_CLASS, text, ws,
		MAIL_BODY_HMARGIN, MAIL_BODY_TOP,
		r.right-MAIL_BODY_HMARGIN, r.bottom - MAIL_BODY_TOP,
		m_hDlgWnd, NULL, m_hInstance, NULL);

	// EChEvV[Wu
	m_pOriginEdit = (WNDPROC)GetWindowLongPtr(hEdit, GWL_WNDPROC);
	SetWindowLongPtr(hEdit, GWL_WNDPROC, (LONG_PTR)CustomEdit);

	// tHgw
	::SendMessage(hEdit, WM_SETFONT, (WPARAM)m_hFont, FALSE);

	::ShowWindow(hEdit, SW_SHOWNA);
	return hEdit;
}

// tHgXV
void CPreviewDlg::FontUpdated()
{
	HFONT hFont = CreateFont();
	::SendMessage(m_hBody, WM_SETFONT, (WPARAM)hFont, TRUE);
	DeleteObject(m_hFont);

	m_hFont = hFont;
}

void CPreviewDlg::RunMUA()
{
	NSM_SERVICE.CallService(RA_RUN_MUA, m_id);
}

void CPreviewDlg::DeleteMail()
{
	NSM_SERVICE.CallService(RA_DELETE_MAIL, m_id, (LPARAM)m_hDlgWnd);
}

//******************************************************************
// 
//******************************************************************

// [{p[gƂɕ
void CPreviewDlg::InitPart()
{
	std::string boundary;

	GetPartBoundary(m_body, boundary);

	if( boundary.length() )
	// }`p[g
	{
		ProcessMultiPart(boundary);
	}
	else
	// VOp[g
	{
		ProcessSinglePart();
	}
}

void CPreviewDlg::ProcessMultiPart(std::string &boundary)
{
	int header_len;
	LPCSTR body_begin;
	LPCSTR first_boundary;

	// wb_vZ
	header_len = GetHeaderLength(m_body.c_str());

	body_begin = m_body.c_str() + header_len;
	first_boundary = strstr(body_begin, boundary.c_str());
	if( first_boundary != NULL ){
		header_len = first_boundary - m_body.c_str();
	}else{
		header_len = strlen(body_begin) + (body_begin - m_body.c_str());
	}

	// [wb_Zbg
	AddPartString(CMB_PREFIX_HEADER);
	m_parts.push_back(NULL_STRING);
	m_parts.back().assign(m_body.c_str(), header_len);

	if( first_boundary != NULL ){
		// [{fB
		char buf[32];
		int len;
		int part_count = 1;
		LPCSTR st = first_boundary;
		std::string pname, pbody;

		strcpy_s(buf, CMB_PREFIX_MULTI);
		while( (len = GetNextPart(st, boundary)) >= 0 ){
			ProcessPart(pname, pbody, st, len);

			_itoa_s(part_count++, buf + LEN_OF(CMB_PREFIX_MULTI), sizeof(buf) - LEN_OF(CMB_PREFIX_MULTI), 10);
			strcat_s(buf, CMB_DELIMITER);
			AddPartString((buf + pname).c_str());
			m_parts.push_back(pbody);

			st += len;
		}
	}
}

void CPreviewDlg::ProcessSinglePart()
{
	int header_len;

	header_len = GetHeaderLength(m_body.c_str());

	// [wb_Zbg
	AddPartString(CMB_PREFIX_HEADER);
	m_parts.push_back(NULL_STRING);
	m_parts.back().assign(m_body.c_str(), header_len);

	if( header_len + LEN_OF(CRLF_STRING) < (int)m_body.length() ){
		std::string pname, pbody;

		// [{fB
		ProcessPart(pname, pbody, m_body.c_str(), m_body.length());

		AddPartString((CMB_PREFIX_SINGLE + pname).c_str());
		m_parts.push_back(pbody);
	}
}

// }`p[goE_̎擾
void CPreviewDlg::GetPartBoundary(const std::string &in_body, std::string &out_booundary) const
{
	std::string c_type;
	LPCSTR st, ed;
	int header_len;

	// Content-type擾ă}`p[gǂf
	header_len = GetHeaderLength(in_body.c_str());
	ExtractHeaderField(in_body.c_str(), HEADER_CONTENT_TYPE, c_type);
	if( FieldCmp(c_type, TYPE_MULTIPART) != 0 ) return;

	// Content-typeboundaryT
	int pos = FieldSearch(c_type, FIELD_BOUNDARY);
	if( pos < 0 ) return;

	// boundary𒊏o
	st = c_type.c_str() + pos + LEN_OF(FIELD_BOUNDARY);
	if( *st == '"' ) st++;
	ed = c_type.c_str() + c_type.length() - 1;
	while( (st < ed) && ((*ed == ' ') || (*ed == '\t') || (*ed == '"')) )
		ed--;

	out_booundary = BOUNDARY_PREFIX;
	out_booundary.append(st, ed - st + 1);
}

// ̃p[gTA̒Ԃ
//  part ݂͌̃p[ĝǂĂB
//  ԂlF -2 I[
//           -1 K؂I[
//           >0 Ԓl part 玟̃oE_̓܂ł̃TCY
//  0ȏ̕ԂlȂ part ͎̃p[g̍ŏ̕Ă
int CPreviewDlg::GetNextPart(LPCSTR &part, const std::string &b) const
{
	LPCSTR st;

	// ݂̃p[gƎ̃p[g̊Ԃ̃oE_T
	st = strstr(part, b.c_str());
	if( st == NULL )
		return NO_NEXT_PART;

	st += b.length();

	// I[oE_Ȃ琳I
	if( strncmp(st, BOUNDARY_POSTFIX, LEN_OF(BOUNDARY_POSTFIX)) == 0 )
		return END_OF_MPART;

	// oE_̂Ƃɂ͉sKv
	st = strstr(st, CRLF_STRING);
	if( st == NULL )
		return NO_NEXT_PART;

	part = st + LEN_OF(CRLF_STRING);

	// ̃oE_T
	st = strstr(part, b.c_str());

	// Ȃ΂邾AȂ炻ꂾ
	return (st == NULL) ? strlen(part): st - part;
}

// X̃p[g
void CPreviewDlg::ProcessPart(std::string &out_pname, std::string &out_pbody, 
		LPCSTR in_part, int len) const
{
	int header_len, body_len, buf_len;
	LPCSTR body_st, buf, buf_ed;
	std::string c_type;
	std::string c_disposition;
	std::string c_encoding;

	header_len = GetHeaderLength(in_part);
	body_len = len - header_len - LEN_OF(CRLF_STRING);
	if( body_len < 0 ) body_len = 0;
	body_st = in_part + len - body_len;

	// obt@쐬
	DynamicArray<char> bin_decoded(body_len+1);
	DynamicArray<char> chr_decoded(body_len+1);

	// wb_tB[h擾
	ExtractHeaderField(in_part, HEADER_CONTENT_TYPE, c_type);
	ExtractHeaderField(in_part, HEADER_CONTENT_DISPOSTION, c_disposition);
	ExtractHeaderField(in_part, HEADER_CONTENT_TRANSFER_ENCODING, c_encoding);

	// 7bitfR[h
	if( FieldCmp(c_encoding, ENC_BASE64) == 0 ){
		buf_ed = DecodeBase64(bin_decoded, body_st, body_st+body_len);
		buf = bin_decoded;
		buf_len = buf_ed - buf;
	}else if( FieldCmp(c_encoding, ENC_QUOTED) == 0 ){
		buf_ed = DecodeQuoted(bin_decoded, body_st, body_st+body_len);
		buf = bin_decoded;
		buf_len = buf_ed - buf;
	}else{
		buf = body_st;
		buf_len = body_len;
	}

	// ̑fR[hݒ
	if( (c_type.length() == 0) || (FieldCmp(c_type, TYPE_TEXT) == 0) )
	// eLXg̏ꍇ
	{
		if( FieldSearch(c_type, FIELD_CHARSET_JIS) >=0 ){
			buf_ed = JIStoSJIS(chr_decoded, buf, buf+buf_len);
			buf = chr_decoded;
			buf_len = buf_ed - buf;
		}else if( FieldSearch(c_type, FIELD_CHARSET_UTF7) >= 0 ){
			std::wstring wstr;
			UTF7toUnicode(buf, wstr);
			UnicodeToSJIS(wstr.c_str(), out_pbody);
			return;
		}else if( FieldSearch(c_type, FIELD_CHARSET_UTF8) >= 0 ){
			std::wstring wstr;
			UTF8toUnicode(buf, wstr);
			UnicodeToSJIS(wstr.c_str(), out_pbody);
			return;
		}

		out_pbody.assign(buf, buf_len);
	}
	else
	// eLXgłȂꍇ
	{
		if( buf_len > BIN_DATA_MAX_SIZE ) buf_len = BIN_DATA_MAX_SIZE;
		for( int i = 0; i <= buf_len; i++ )
			chr_decoded[i] = (buf[i] == '\0') ? ' ': buf[i];

		out_pbody.assign(in_part, header_len);
		out_pbody += CRLF_STRING;
		out_pbody.append(chr_decoded, buf_len);
	}

	// p[gݒ
	{
		int pos;

		if( (pos = FieldSearch(c_disposition, FIELD_FILENAME_PREFIX)) >= 0 ){
			GetAttName(c_disposition.c_str() + pos + LEN_OF(FIELD_FILENAME_PREFIX), out_pname);
		}else if( (pos = FieldSearch(c_type, FIELD_NAME_PREFIX)) >= 0 ){
			GetAttName(c_type.c_str() + pos + LEN_OF(FIELD_NAME_PREFIX), out_pname);
		}else{
			out_pname.assign(c_type, 0, c_type.find(';'));
		}
	}
}

void CPreviewDlg::GetAttName(LPCSTR in_fvalue, std::string &out_fname) const
{
	if( *in_fvalue != '*' )
	// MIME or oCȉꍇ
	// filename=
	{
		std::string fname;
		size_t len;

		in_fvalue++;

		if( *(in_fvalue) == '"' ){
			CutOutFieldValue(in_fvalue+1, fname);
			len = fname.find('"');
		}else{
			CutOutFieldValue(in_fvalue, fname);
			len = fname.find(';');
		}

		if( len != std::string::npos )
			fname.resize(len);

		Decode(fname, out_fname);
	}
	else
	// RFC2231̏ꍇ
		// filename*=
		// filename*0=
		// filename*0*=
	{
		LPCSTR pos, st, ed;
		bool bJis = false;
		bool bMultiLine;
		DynamicArray<char> asc_buf(strlen(in_fvalue+1));
		DynamicArray<char> sj_buf(strlen(in_fvalue+1));
		LPSTR buf;

		// sH
		bMultiLine = (*(in_fvalue + 1) != '=');

		// t@CJnʒu
		st = strchr(in_fvalue, '=');
		if( st == NULL ) return;

		ed = strchr(st, ';');
		if( ed == NULL ) ed = in_fvalue + strlen(in_fvalue);

		if( *(st-1) == '*' ){
			bJis = (_strnicmp(st+1, ISO_2022_JP, LEN_OF(ISO_2022_JP)) == 0);

			pos = in_fvalue - 1;
			while( pos != NULL ){
				st = pos + 1;
				pos = (LPCSTR)memchr(st, '\'', ed - st);
			}
		}
 
		buf = asc_buf;
		buf = DecodeURL(buf, st, ed);

		if( bMultiLine ){
			for(;;){
				st = strstr(ed, FIELD_FILENAME2231_PREFIX);
				if( st == NULL ) break;
				st = strchr(st, '=');
				if( st == NULL ) break;
				ed = strchr(st, ';');
				if( ed == NULL ) ed = st + strlen(st);

				if( *(st - 1) == '*' ){
					buf = DecodeURL(buf, st + 1, ed);
				}else{
					buf = Unquote(buf, st + 1, ed);
				}
			}
		}

		if( bJis ){
			JIStoSJIS(sj_buf, asc_buf, buf);
			out_fname = sj_buf;
		}else{
			out_fname = asc_buf;
		}
	}
}

//******************************************************************
// static
//******************************************************************

// NbJuURLpGfBbg{bNXEBhEvV[W
LRESULT CALLBACK CPreviewDlg::CustomEdit(HWND hWnd, UINT msg, WPARAM wParam, LPARAM lParam)
{
	if( msg == WM_LBUTTONDBLCLK ){
		if( URLHandling(hWnd) )
			return 0;
	}
	return CallWindowProc(m_pOriginEdit, hWnd, msg, wParam, lParam);
}

// NbJuURL̎
bool CPreviewDlg::URLHandling(HWND hWnd)
{
	DWORD n_st, n_ed;
	LPSTR p_st, p_ed;
	LPSTR buf_ed, sel_pos;
	LPCSTR pos;
	u_int len;


	// GfBbg{bNXeLXg̎擾
	len = ::GetWindowTextLength(hWnd);
	DynamicArray<char> buf(len+1);
	::GetWindowText(hWnd, buf, len);
	buf[len] = '\0';

	// J[\ʒu̎擾
	{
		::SendMessage(hWnd, EM_GETSEL, (WPARAM)&n_st, NULL);

		buf_ed = buf + len;
/*		if( ::IsWindowUnicode(hWnd) )
		{
			for( sel_pos = buf; (n_st > 0) && (sel_pos < buf_ed); n_st-- )
				sel_pos = (char*)_mbsinc((const unsigned char*)sel_pos);
		}
		else
*/		{
			if( n_st > len ){
				return false;
			}
			sel_pos = buf + n_st;
		}
	}

	// J[\t߂URLۂ̎擾
	{
		for( p_st = sel_pos ; buf <= p_st; p_st-- )
			if( ! IsURLChar(*p_st) ) break;

		for( p_ed = sel_pos; p_ed < buf_ed; p_ed++ )
			if( ! IsURLChar(*p_ed) ) break;
		*p_ed = '\0';
	}

	// URLۂJĂ݂
	if( (pos = TryOpen(p_st, URL_HTTP)) == NULL )
		if( (pos = TryOpen(p_st, URL_HTTPS)) == NULL )
			if( (pos = TryOpen(p_st, URL_FTP)) == NULL )
				if( (pos = TryOpen(p_st, PATH_REMOTE)) == NULL ){
					return false; // JȂ
				}

	// URLuIv
	n_st = 0;
//	n_ed = _mbslen((const unsigned char*)(char*)buf);
	n_ed = strlen(buf);

	if( buf < pos ){
//		*pos = '\0';
//		n_st = _mbslen((const unsigned char*)(char*)buf);
//		n_st = strlen(buf);
		n_st = pos - buf;
	}
	::SendMessage(hWnd, EM_SETSEL, n_st, n_ed);

	return true;
}

// URLɎgp\ȕǂ
bool CPreviewDlg::IsURLChar(char c)
{
	return  ! (('~' < c) || (c <= ' ') || (c == '"') || (c == '\'')
		|| (c == '(') || (c == ')') || (c == '<') || (c == '>'));
}

// stɕschema܂܂Ă炻ȍ~uJv
LPCSTR CPreviewDlg::TryOpen(LPCSTR st, LPCSTR schema)
{
	_ASSERT( st != NULL );
	_ASSERT( schema != NULL );
	LPCSTR pos = strstr(st, schema);

	if( pos != NULL ){
		if( (int)::ShellExecute(NULL, NULL, pos, NULL, NULL, SW_SHOWNORMAL) > 32 )
			return pos;
		CRiffNsm::NotifyError("URL̓pX͊J܂ł");
	}
	return NULL;
}
