#include "StdAfx.h"
#include <process.h>
#include <time.h>
#include <fstream>
#include <algorithm>
#include ".\riffmain.h"
#include "codec.h"
#include "../nsmsgs/util.h"
#include "../nsmsgs/DynamicArray.h"
#include "../nsmsgs/mleak.h"

HANDLE CRiffMain::m_end = INVALID_HANDLE_VALUE;
int CRiffMain::m_active = 0;
CRITICAL_SECTION CRiffMain::m_cs;

//******************************************************************
// 
//******************************************************************

CRiffMain::CRiffMain(HNsmConnection hConnection, LPCSTR lpAcc)
: m_nsm(hConnection, lpAcc)
{
	Enter();
	LoadConfig(lpAcc);
	LoadMails(m_mails);

	switch( m_set.protocol ){
	case RP_POP3: m_pServer = new CPop3Server(lpAcc, &m_nsm); break;
//	case RP_IMAP4: m_pServer = new CImap4Server(lpAcc); break;
	default:
		_ASSERT(FALSE);
		break;
	}


	// ƃXbh쐬
	m_timerID = NULL;
	m_hThreadRdyEvt = CreateEvent(NULL, TRUE, FALSE, NULL);
	_beginthread(RiffThread, 0, this);
	WaitForSingleObject(m_hThreadRdyEvt, INFINITE);
	CloseHandle(m_hThreadRdyEvt);
}

CRiffMain::~CRiffMain(void)
{
	UnloadMails(m_mails);
	delete m_pServer;
	Leave();
}

//******************************************************************
// 
//******************************************************************

void CRiffMain::BeginCheck(void)
{
	_ASSERT(m_threadID != NULL);
	::PostThreadMessage(m_threadID, WM_USER_BEGIN_CHECK, 0, 0);
}

void CRiffMain::EndCheck(void)
{
	_ASSERT(m_threadID != NULL);
	::PostThreadMessage(m_threadID, WM_USER_END_CHECK, 0, 0);
}

void CRiffMain::CheckNow(void)
{
	_ASSERT(m_threadID != NULL);
	::PostThreadMessage(m_threadID, WM_USER_CHECK_NOW, 0, 0);
}

void CRiffMain::ConfigChanged(LPCSTR lpAcc)
{
	_ASSERT(m_threadID != NULL);
	::PostThreadMessage(m_threadID, WM_USER_CONFIG_UPDATED, 0, (LPARAM)::DuplicateString(lpAcc));
}

void CRiffMain::DeleteMail(LPCSTR gid)
{
	_ASSERT(m_threadID != NULL);
	::PostThreadMessage(m_threadID, WM_USER_DELETE_MAIL, 0, (LPARAM)::DuplicateString(gid));
}

void CRiffMain::ViewMail(LPCSTR gid)
{
	_ASSERT(m_threadID != NULL);
	::PostThreadMessage(m_threadID, WM_USER_VIEW_MAIL, 0,  (LPARAM)::DuplicateString(gid));
}

void CRiffMain::Exit()
{
	_ASSERT(m_threadID != NULL);
	::PostThreadMessage(m_threadID, WM_QUIT, 0, 0);
}

//******************************************************************
// 
//******************************************************************

void CRiffMain::DoBeginCheck(void)
{
	if( m_timerID != NULL ) return;

	m_nsm.BeginCheck();
	m_timerID = ::SetTimer(NULL, NULL, m_set.interval * 1000, NULL);
}

void CRiffMain::DoEndCheck(void)
{
	if( m_timerID == NULL ) return;

	::KillTimer(NULL, m_timerID);
	m_timerID = NULL;
	m_nsm.EndCheck();
}

void CRiffMain::DoCheckNow(void)
{
	MailMap::iterator mail;
	MailIDMap list, add_ids;
	MailIDMap::reverse_iterator id;
	std::string header;
	int count = 0;
	int getnum;
	char buf[64];
	MailMap add, del;
	CMailInfo *pmi;
	time_t current_time;

	time(&current_time);

	try{
		m_nsm.NotifyState("T[o֐ڑ");
		m_pServer->Connect();

		m_nsm.NotifyState("[ꗗ擾");
		m_pServer->GetGIDList(list);

		m_nsm.NotifyState("𒊏o");
		ExtractMailDiff(list, add_ids, del);

		// V[̏擾
		getnum = min(add_ids.size(), (u_int)m_set.max_mail);
		for( id = add_ids.rbegin(); id != add_ids.rend(); id++ )
		{
			pmi = new CMailInfo(id->first.c_str(), current_time);

			if( count++ < m_set.max_mail ){
				sprintf(buf, "[wb_擾 %d/%d", count, getnum);
				m_nsm.NotifyState(buf);

				m_pServer->GetMailHeader(id->second, header);
				pmi->SetFromHeader(header.c_str());
			}
			add[pmi->gid()] = pmi;
		}

		m_pServer->Disconnect();
	}catch(LPCSTR msg){
		if( m_pServer->IsConnected() )
			m_pServer->Disconnect();
		m_nsm.Error(msg);
	}

	// 폜ꂽ[[Xg폜
	for( mail = del.begin(); mail != del.end(); mail++ ){
		m_mails.erase(mail->first);
	}

	m_nsm.NotifyState("\XV");
	m_nsm.MailUpdate(m_mails, add, del);

	// 폜ꂽ[̃[J
	UnloadMails(del);

	// V[[Xgɒǉ
	m_mails.insert(add.begin(), add.end());

	SaveMails(m_mails);
	m_nsm.NotifyDefaultStatus();
}

void CRiffMain::DoConfigChanged(LPCSTR lpAcc)
{
	LoadConfig(lpAcc);
	m_pServer->ConfigChanged(lpAcc);
	m_nsm.ConfigChanged(lpAcc);
	EndCheck();
	BeginCheck();
}

void CRiffMain::DoDeleteMail(LPCSTR gid)
{
	try{
		m_pServer->Connect();
		m_pServer->DeleteMail(GIDtoMailNo(gid));
		m_pServer->Disconnect();
	}catch(LPCSTR msg){
		m_nsm.Error(msg);
	}

	m_nsm.DeleteMail(gid);
	m_mails.erase(gid);
}

void CRiffMain::DoViewMail(LPCSTR gid)
{
	std::string mail;
	int pos;
	LPSTR body = NULL;

	try{
		m_nsm.NotifyState("ڑ");
		m_pServer->Connect();

		m_nsm.NotifyState("[̎擾");
		m_pServer->GetMail(GIDtoMailNo(gid), mail, m_set.preview_linenum);

		m_nsm.NotifyState("ؒf");
		m_pServer->Disconnect();

		m_nsm.NotifyState("[ϊ");
		pos = mail.find("\r\n\r\n");
		if( pos != -1 ){
			body = new char[mail.length() - pos];
			JIStoSJIS(mail.c_str() + pos + 4, body, mail.length() - pos - 4);
			mail.resize(pos + 2);
		}
		m_mails[gid]->SetPreviewed();
		m_nsm.ViewMail(*m_mails[gid], body ? body: "");
		SaveMails(m_mails);
	}catch(LPCSTR msg){
		m_nsm.Error(msg);
	}

	delete body;
	m_nsm.NotifyDefaultStatus();
}

//******************************************************************
// 
//******************************************************************

void CRiffMain::ExtractMailDiff(const MailIDMap &ids,
		MailIDMap &add, MailMap &del) const
{
	MailMap::const_iterator mail;
	MailIDMap::const_iterator id;

	// 폜ꂽ[𒊏o
	for( mail = m_mails.begin(); mail != m_mails.end(); mail++ ){
		if( ids.find(mail->first) == ids.end() ){
			del[mail->first] = mail->second;
		}
	}

	// ǉꂽ[𒊏o
	for( id = ids.begin(); id != ids.end(); id++ ){
		if( m_mails.find(id->first.c_str()) == m_mails.end() ){
			add[id->first] = id->second;
		}
	}
}

void CRiffMain::LoadMails(MailMap &prevs) const
{
	std::ifstream f(m_gid_filename.c_str());
	CMailInfo *pmi;

	if( f.fail() ) return;

	while( ! f.eof() ){
		pmi = new CMailInfo;

		*pmi << f;

		if( f.fail() ){
			delete pmi;
			return;
		}
		prevs[pmi->gid()] = pmi;
	}
}

void CRiffMain::SaveMails(MailMap &prevs) const
{
	MailMap::const_iterator i;

	std::ofstream f(m_gid_filename.c_str());
	if( f.fail() ) return;

	for( i = prevs.begin(); i != prevs.end(); i++ ){
		*(i->second) >> f;
	}
}

void CRiffMain::UnloadMails(MailMap &prevs) const
{
	MailMap::iterator i;

	for( i = prevs.begin(); i != prevs.end(); i++ ){
		delete i->second;
	}
}

int CRiffMain::GIDtoMailNo(LPCSTR gid)
{
	_ASSERT( m_pServer->IsConnected() );
	MailIDMap list;
	MailIDMap::iterator id;

	m_pServer->GetGIDList(list);
	id = list.find(gid);
	if( id == list.end() ){
		throw "w肳ꂽ[݂܂";
	}
	return id->second;
}

//******************************************************************
// 
//******************************************************************

void __cdecl CRiffMain::RiffThread(void* param)
{
	CRiffMain &rThis = *(CRiffMain*)param;
	MSG msg;
	LPCSTR str;

	// bZ[WL[쐬
	rThis.m_threadID = ::GetCurrentThreadId();
	PeekMessage(&msg, NULL, WM_USER, WM_USER, PM_NOREMOVE);
	SetEvent(rThis.m_hThreadRdyEvt);

	while ( ::GetMessage(&msg, NULL, 0, 0) > 0 )
    {
		str = (LPCSTR)msg.lParam;

		switch(msg.message){
		case WM_TIMER:
			rThis.DoCheckNow();
			break;
		case WM_USER_VIEW_MAIL:
			rThis.DoViewMail(str);
			delete str;
			break;
		case WM_USER_CHECK_NOW:
			rThis.DoCheckNow();
			break;
		case WM_USER_DELETE_MAIL:
			rThis.DoDeleteMail(str);
			delete str;
			break;
		case WM_USER_CONFIG_UPDATED:
			rThis.DoConfigChanged(str);
			delete str;
			break;
		case WM_USER_BEGIN_CHECK:
			rThis.DoBeginCheck();
			rThis.CheckNow();
			break;
		case WM_USER_END_CHECK:
			rThis.DoEndCheck();
			break;
		case WM_USER_EXIT:
			return;
		default:
			break;
		}
	}

	delete &rThis;
}
