#pragma once
#include <string>
#include <map>
#include <functional>
#include "RiffNsm.h"
#include "MailServer.h"
#include "RiffSettings.h"
#include "Pop3Server.h"
//#include "Imap4Server.h"
#include "types.h"


//**************************************************
#define SERVER_STR_LEN	128
#define USER_STR_LEN	16
#define PASS_STR_LEN	16

struct RiffInfo
{
	char server[SERVER_STR_LEN];
	unsigned short port;
	char user[USER_STR_LEN];
	char pass[PASS_STR_LEN];
	int auth_type;
	bool keep_con;
	int interval;
	int display_term;
	int max_display;
};

//**************************************************
class CRiffMain
{
private:
	enum
	{
		WM_USER_EXIT = WM_USER+1,
		WM_USER_CHECK_NOW,
		WM_USER_CONFIG_UPDATED,
		WM_USER_DELETE_MAIL,
		WM_USER_VIEW_MAIL,
		WM_USER_BEGIN_CHECK,
		WM_USER_END_CHECK,
	};

private:
	CRiffNsm m_nsm;
	CMailServer *m_pServer;
	AccSettings m_set;
	DWORD m_threadID;
	HANDLE m_hThreadRdyEvt;
	UINT_PTR m_timerID;
	std::string m_gid_filename;
	MailMap m_mails;

private:
	~CRiffMain(void);
	static HANDLE m_end;
	static CRITICAL_SECTION m_cs;
	static int m_active;
	static void Enter()
	{
		EnterCriticalSection(&m_cs);
		if( m_active <= 0 ) ResetEvent(m_end);
		m_active++;
		LeaveCriticalSection(&m_cs);
	}
	static void Leave()
	{
		EnterCriticalSection(&m_cs);
		m_active--;
		if( m_active <= 0 ) SetEvent(m_end);
		LeaveCriticalSection(&m_cs);
	}

public:
	CRiffMain(HNsmConnection hConnection, LPCSTR lpAcc);

public:
	void BeginCheck(void);
	void EndCheck(void);
	void CheckNow(void);
	void ConfigChanged(LPCSTR lpAcc);
	void DeleteMail(LPCSTR gid);
	void ViewMail(LPCSTR gid);
	void Exit();
	static void Init()
	{
		InitializeCriticalSection(&m_cs);
		m_end = CreateEvent(NULL, FALSE, TRUE, NULL);
		_ASSERT(m_end != INVALID_HANDLE_VALUE);
	}
	static void Cleanup()
	{
		WaitForSingleObject(m_end, INFINITE);
		CloseHandle(m_end);
		DeleteCriticalSection(&m_cs);
	}

private:
	void DoBeginCheck(void);
	void DoEndCheck(void);
	void DoCheckNow(void);
	void DoConfigChanged(LPCSTR lpAcc);
	void DoDeleteMail(LPCSTR gid);
	void DoViewMail(LPCSTR gid);

	int GIDtoMailNo(LPCSTR gid);
	void ExtractMailDiff(const MailIDMap &ids,
		MailIDMap &add, MailMap &del) const;
	void LoadMails(MailMap &prevs) const;
	void SaveMails(MailMap &prevs) const;
	void UnloadMails(MailMap &prevs) const;
	void LoadConfig(LPCSTR lpAcc)
	{
		RIFF_SETTINGS.GetGIDFileName(lpAcc, m_gid_filename);
		RIFF_SETTINGS.GetAccSettings(lpAcc, m_set);
	}

private:
	static void __cdecl RiffThread(void* param);
};
