#include "StdAfx.h"
#include "service_name.h"
#include ".\riffnsm.h"
#include "RiffSettings.h"
#include "general_strings.h"
#include "mleak.h"

HNsmEvent CRiffNsm::ms_hLogEvt = NULL;

const LPWSTR CRiffNsm::MAIL_STATUS[] = {
	L"",
	L"vr[",
	L"vr[",
	L"ŋ߂̃[",
	L"V",
};

const static LPCSTR HEADER_FROM		= "M: ";
const static LPCSTR HEADER_TO		= ": ";
const static LPCSTR HEADER_DATE		= "M: ";
const static LPCSTR HEADER_SUBJECT	= ": ";

//******************************************************************
// 
//******************************************************************

CRiffNsm::CRiffNsm(LPCSTR lpAcc)
: m_hCon( NSM_SERVICE.CreateConnection(THIS_PROTOCOL, lpAcc) )
, m_nProx(0)
, m_nAutoCheckOnProx(0)
, m_bMark(false)
{
	for( int i = 0; i < NUM_OF_NUM; i++ ) m_mnums[i] = 0;
	NSM_SERVICE.SetConnectionInfo(m_hCon, NMCI_STATUS, NMCS_CONNECTING);
	NSM_SERVICE.SetConnectionInfo(m_hCon, NMCI_USER_STATUS, NMST_CUSTOM);
	NSM_SERVICE.SetConnectionInfo(m_hCon, NMCI_USER_ACCOUNT, lpAcc);
}

CRiffNsm::~CRiffNsm(void)
{
	NSM_SERVICE.DeleteConnection(m_hCon);
}

//******************************************************************
// 
//******************************************************************

bool CRiffNsm::AddMail(int id, const CMailInfo &mi, MailStatus s) const
{
	int ret;
	std::string info;

	ret = NSM_SERVICE.CreateConnectionMember(m_hCon, mi.gids());
	if( ret == 0 ) return false;

	InfoToText(mi, info);
	NSM_SERVICE.SetConnectionMemberInfo(m_hCon, mi.gids(), NMMI_NAME, mi.subject());
	NSM_SERVICE.SetConnectionMemberInfo(m_hCon, mi.gids(), NMMI_HINT, info.c_str());
	NSM_SERVICE.SetConnectionMemberInfo(m_hCon, mi.gids(), NMMI_STATUS,
									(s == RM_PREVIEWED) ? NMST_OFFLINE: NMST_ONLINE);
//	NSM_SERVICE.SetConnectionMemberInfo(m_hCon, mi.gids(), NMMI_STATUSSTRING, MAIL_STATUS[s]);

	SetMailStatus(id, mi.gids());
	ChangeMailStatus(mi.gids(), s);
	return true;
}

// [폜
void CRiffNsm::DeleteMail(LPCSTR gid) const
{
	NSM_SERVICE.DeleteConnectionMember(m_hCon, gid);
}

void CRiffNsm::UpdateCheckState() const
{
	char buf[DEF_SIZE];
	LPCSTR state;

	if( m_nAutoCheckOnProx == 0 ){
		state = (m_nProx == 1) ? AUTO_CHECK_OFF: AUTO_CHECK_ALL_OFF;
	}else if( m_nAutoCheckOnProx == m_nProx ){
		state = (m_nProx == 1) ? AUTO_CHECK_ON: AUTO_CHECK_ALL_ON;
	}else{
		sprintf_s(buf, AUTO_CHECK_ON_FORMAT, m_nAutoCheckOnProx, m_nProx);
		state = buf;
	}

// NMCI_USER_STATUSSTRING\ȂȂĂ܂B
//		NSM_SERVICE.SetConnectionInfo(m_hCon, NMCI_USER_STATUS,
//			(m_nAutoCheckOnProx == 0) ? NMST_OFFLINE: NMST_ONLINE);

	NSM_SERVICE.SetConnectionInfo(m_hCon, NMCI_USER_STATUSSTRING, state);
}

void CRiffNsm::MailStatusUpdated(int id, LPCWSTR acc,
	const int old_mnums[], const int new_mnums[])
{
	for( int i = 0; i < NUM_OF_NUM; i++ ){
		m_mnums[i] += new_mnums[i] - old_mnums[i];
		UpdateMark();
	}
	
	MailStatusUpdate(id, acc, old_mnums, new_mnums);
}

void CRiffNsm::UpdateMark()
{
	int num = 0;

	switch( RIFF_SETTINGS.GetMarkType() ){
	case RW_PREVIEW: num += m_mnums[RM_NORMAL];
	case RW_RECENT: num += m_mnums[RM_RECENT];
	case RW_NEW: num += m_mnums[RM_NEW];
	default:
		if( m_bMark && num == 0 ){
			NSM_SERVICE.SetConnectionInfo(m_hCon, NMCI_CAPTION, m_caption.c_str());
			m_bMark = false;
		}else if( ! m_bMark && num != 0 ){
			std::wstring wacc;
			wacc = L"* ";
			wacc += m_caption;
			NSM_SERVICE.SetConnectionInfo(m_hCon, NMCI_CAPTION, wacc.c_str());
			m_bMark = true;
		}
		break;
	}
}

//******************************************************************
// 
//******************************************************************

// [̃c[`bveLXg쐬
void CRiffNsm::InfoToText(const CMailInfo &mi, std::string &str) const
{
	str = HEADER_FROM;
	str += mi.from();
	str += CRLF_STRING;

	str += HEADER_TO;
	str += mi.to();
	str += CRLF_STRING;

	str += HEADER_DATE;
	str += mi.dateStr();
	str += CRLF_STRING;

	str += HEADER_SUBJECT;
	str += mi.subject();
}

//******************************************************************
// 
//******************************************************************

void __stdcall CRiffNsm::ViewServiceCaller(int param)
{
	NSM_SERVICE.CallService(RA_PREVIEW_MAIL, param);
}

void __stdcall CRiffNsm::MailerServiceCaller(int param)
{
	NSM_SERVICE.CallService(RA_RUN_MUA, param);
}
