#pragma once
#include <string>
#include <sstream>
#include "ComLog.h"
#include "RiffGlobal.h"
#include "mail_info.h"
#include "CommonService.h"
#include "general_strings.h"
#include "util.h"
#include "cs_conv.h"

//***********************************************************

const static LPCSTR AUTO_CHECK_OFF			= "`FbNOFF";
const static LPCSTR AUTO_CHECK_ON			= "`FbNON";
const static LPCSTR AUTO_CHECK_ALL_OFF		= "`FbNSOFF";
const static LPCSTR AUTO_CHECK_ALL_ON		= "`FbNSON";
const static LPCSTR AUTO_CHECK_ON_FORMAT	= "`FbN(%d/%d)ON";


class CRiffNsm: public CComLog
{
// 萔
public:
	const static HNsmConnection ALL_CONNECTIONS = 0;

protected:
	enum{ DEF_SIZE=64 };

protected:
	const static LPWSTR MAIL_STATUS[NUM_OF_NUM];
	const HNsmConnection m_hCon;

// oϐ
private:
	static HNsmEvent ms_hLogEvt;
	int m_nProx;
	int m_nAutoCheckOnProx;
	int m_mnums[NUM_OF_NUM];
	bool m_bMark;
	std::wstring m_caption;

// o֐
public:
	CRiffNsm(LPCSTR lpAcc);
	const CRiffNsm& operator =(const CRiffNsm&)
	{ _ASSERT(FALSE); }
	virtual ~CRiffNsm(void);

public:
	bool AddMail(int id, const CMailInfo &mi, MailStatus s) const;
	void DeleteMail(LPCSTR gid) const;
	void UpdateCheckState() const;
	void MailStatusUpdated(int id, LPCWSTR acc,
		const int old_mnums[], const int new_mnums[]);

	virtual void UpdateProx(int id, LPCWSTR acc, int mnums[]) = 0;
	virtual void NotifyState(int id, LPCWSTR acc, LPCSTR msg) const=  0;
	virtual void NotifyDefaultState(int id, LPCWSTR acc, const int mnums[]) const = 0;
	virtual void ChangeMailStatus(LPCSTR gid, MailStatus s) const = 0;
	virtual void MailStatusUpdate(int id, LPCWSTR acc,
		const int old_mnums[], const int new_mnums[]) const = 0;

	HNsmConnection GetConnection() const
	{ return m_hCon; }
	virtual void RegistProx(int /*id*/, LPCWSTR /*acc*/)
	{ m_nProx++; }
	void BeginCheck()
	{
		Lock();
		m_nAutoCheckOnProx++;
		UpdateCheckState();
		Unlock();
	}
	void EndCheck()
	{
		Lock();
		m_nAutoCheckOnProx--;
		UpdateCheckState();
		Unlock();
	}
	bool ConfirmDelete(const CMailInfo &mi, HWND hParent) const
	{
		std::ostringstream oss;
		std::wstring wstr;
		bool bYes;

		oss << ": " << mi.subject() << CRLF_STRING;
		oss << ": " << mi.from() << CRLF_STRING;
		oss << "ȏ̃[폜Ă낵łH";
		::SJIStoUnicode(oss.str().c_str(), wstr);
		bYes = (NSM_SERVICE.MsgBox(L"mF", wstr.c_str(), MB_YESNO, hParent) == IDYES);

		if( bYes && (hParent != NULL) ) ::CloseWindow(hParent);
		return bYes;
	}
	void DoLog(LPCSTR header, LPCSTR body) const
	{
		TNsmComLogInfo ncli;

		ncli.cbSize = sizeof(ncli);
		ncli.nType = NMHT_CONNECTION;
		ncli.lpHandle = m_hCon;
		ncli.lpHeader = header;
		ncli.lpMessage = body;
		NSM_SERVICE.NotifyEvent(ms_hLogEvt, (WPARAM)&ncli);
	}
	void Error(CRiffException e)
	{
		std::ostringstream errmsg;
		std::wstring wstr;
		std::string logstr;
		
		FormatErrMsg(errmsg, e);

		::SJIStoUnicode(errmsg.str().c_str(), wstr);
		::UnicodeToUTF8(wstr.c_str(), logstr);
		
		DoLog("-", logstr.c_str());
		CRiffNsm::Notify(true, errmsg.str().c_str(), NMNI_STOP);
	}

public:
	static void __stdcall ViewServiceCaller(int param);
	static void __stdcall MailerServiceCaller(int param);
	static void  SetLogEventHandle(HNsmEvent hEvt)
	{ ms_hLogEvt = hEvt; }
	static bool Notify(bool bForce, LPCTSTR lpMsg, NotifyIconType icon=NMNI_EXCLAMATION,
		int time=-1, TNotifyClickCallback cback=NULL, int nData=NULL)
	{
		// UIftHgƖ͂ǂ悤Ȃ̂ 10 Ō߂
		int chk_time = (time <= 0) ? 10: time;

		if( RIFF_GLOBAL.IsOverNotifyNum(bForce, chk_time) ) return false;
		NSM_SERVICE.DefaultNotify(lpMsg, icon, time, cback, nData);
		return true;
	}
	static void NotifyError(CRiffException e)
	{
		std::ostringstream errmsg;
		FormatErrMsg(errmsg, e);
		CRiffNsm::Notify(true, errmsg.str().c_str(), NMNI_STOP);
	}

protected:
	virtual void Lock() = 0;
	virtual void Unlock() = 0;
	virtual void SetMailStatus(int id, LPCSTR gid) const = 0;
	void InfoToText(const CMailInfo &mi, std::string &str) const;
	void UpdateMark();
	void SetCaption(LPCWSTR cap)
	{ m_caption = cap; }
	static void FormatErrMsg(std::ostringstream &errmsg, CRiffException& e)
	{
		errmsg << e.GetErrorMsg() << CRLF_STRING;
		errmsg << "G[R[hF" << e.GetLastError();
		errmsg << " / " << e.WSAGetLastError();

		if( e.ERR_get_error() != 0 )
		{
			char buf[1024];
			ERR_error_string_n(e.ERR_get_error(), buf, sizeof(buf));
			errmsg << CRLF_STRING;
			errmsg << buf;
		}
	}
};
