#include "StdAfx.h"
#include <process.h>
#include <time.h>
#include ".\riffnsmprox.h"
#include "service_name.h"
#include "PreviewDlg.h"
#include "RiffGlobal.h"
#include "NsmPlugin.h"
#include "mleak.h"

//******************************************************************
// 
//******************************************************************

CRiffNsmProx::CRiffNsmProx(CRiffNsm *pnsm, CGID gid, UISettings &set)
: m_pNsm(pnsm)
, m_gid(gid)
, m_set(set)
{
	for( int i = 0; i < NUM_OF_NUM; i++ ) m_mnums[i] = 0;
	::SJIStoUnicode(m_set.account.c_str(), m_wAcc);
	m_pNsm->RegistProx(m_gid.chkr_id(), m_wAcc.c_str());

	if( m_set.bDoLog ){
		std::wstring wstr = m_wAcc + L" Jn";
		std::string ustr;

		UnicodeToUTF8(wstr.c_str(), ustr);
		m_pNsm->DoLog("S", ustr.c_str());
	}
}

CRiffNsmProx::~CRiffNsmProx(void)
{
}

//******************************************************************
// public
//******************************************************************

void CRiffNsmProx::MailSync(const NumberMap &mails)
{
	int old[NUM_OF_NUM];

	CopyMNums(old, m_mnums);
	for( int i = 0; i < NUM_OF_NUM; i++ ) m_mnums[i] = 0;

	MailSyncDelete(mails);
	MailSyncAdd(mails, m_set.max_mail, m_mnums);
//	UpdateMails(mails, m_set.max_mail, m_mnums);
	MailStatusUpdated(old, m_mnums);
}

void CRiffNsmProx::NewMail(const NumberMap &adds)
{
	NumberMap::const_reverse_iterator mail;
//	int mcount = m_set.max_mail;

	for( mail = adds.rbegin(); mail != adds.rend(); ++mail ){
//		if( mcount-- <= 0 ) break;
		const CMailInfo &mi = *(mail->second);

//		AddMail(mi, RM_NEW);
		if( m_set.notify_each ) NotifyMail(mi);
	}

	if( adds.size() > 0 ){
		NotifyNewMailNum(adds.size());
	}
}

/*
void CRiffNsmProx::MailUpdate(const NumberMap &mails, const NumberMap &add, const GIDList &del)
{
	int old[NUM_OF_NUM];

	CopyMNums(old, m_mnums);
	for( int i = 0; i < NUM_OF_NUM; i++ ) m_mnums[i] = 0;

	// 폜ꂽ[̏
	DeleteMails(del);
	
	// ύXɂȂ郁[̏
	UpdateMails(mails, m_set.max_mail - add.size(), m_mnums);

	// V[̏
	AddMails(add);

	m_mnums[RM_NEW] = add.size();
	MailStatusUpdated(old, m_mnums);

	if( add.size() > 0 ){
		NotifyNewMailNum(add.size());
	}
}

void CRiffNsmProx::DeleteMail(const CMailInfo &mi)
{
	StatusMap::iterator i;

	i = m_status.find(mi.gid());
	if( i != m_status.end() ){
		_ASSERT( 0 <= i->second && i->second < NUM_OF_NUM );
		int old[NUM_OF_NUM];

		CopyMNums(old, m_mnums);

		m_mnums[i->second]--;
		MailStatusUpdated(old, m_mnums);
		m_status.erase(i);
	}

	DeleteMail(mi.gid(), mi.gids());
}
*/
void CRiffNsmProx::ViewMail(const CMailInfo &mi, const std::string &body)
{
	CPreviewDlg *pd = new CPreviewDlg();
	StatusMap::iterator i;

	i = m_status.find(mi.gid());
	if( i != m_status.end() ){
		_ASSERT( 0 <= i->second && i->second < NUM_OF_NUM );
		int old[NUM_OF_NUM];

		CopyMNums(old, m_mnums);

		m_mnums[i->second]--;
		m_mnums[RM_PREVIEWED]++;

		m_pNsm->ChangeMailStatus(mi.gids(), RM_PREVIEWED);
		MailStatusUpdated(old, m_mnums);

		i->second = RM_PREVIEWED;
	}
	
	pd->Set(mi.gid(), mi.subject(), mi.from(), mi.to(), mi.dateStr(), body.c_str());

	_beginthread(PreviewThread, 0, pd);
}

void CRiffNsmProx::Error(CRiffException e) const
{
	std::ostringstream accmsg;

	accmsg << "[" << m_set.account << "]\r\n";
	accmsg << e.GetErrorMsg();
	std::string msgstr = accmsg.str();
	e.ReplaceErrorMsg(msgstr.c_str());
	m_pNsm->Error(e); 
}

//******************************************************************
// private
//******************************************************************
/*
void CRiffNsmProx::AddMails(const NumberMap adds)
{
	NumberMap::const_reverse_iterator mail;
	int mcount = m_set.max_mail;

	for( mail = adds.rbegin(); mail != adds.rend(); ++mail ){
		if( mcount-- <= 0 ) break;
		const CMailInfo &mi = *(mail->second);

		AddMail(mi, RM_NEW);
		if( m_set.notify_each ) NotifyMail(mi);
	}
}

void CRiffNsmProx::DeleteMails(const GIDList dels)
{
	StatusMap::iterator i;
	GIDList::const_reverse_iterator rmail;
	char gids[CGID::GIDS_LEN+1];

	for( rmail = dels.rbegin(); rmail != dels.rend(); ++rmail ){
		i = m_status.find(*rmail);
		if( i != m_status.end() ){
			m_status.erase(i);
		}
		rmail->ToStr(gids);
		DeleteMail(*rmail, gids);
	}
}

void CRiffNsmProx::UpdateMails(const NumberMap mails, int maxMail, int mnums[])
{
	NumberMap::const_iterator mail;
	StatusMap::iterator status;
	time_t recent_limit;
	MailStatus cur_s, nxt_s;

	time(&recent_limit);
	recent_limit -= m_set.show_term;

	// 폜ꂽ[̓
	for( status = m_status.begin(); status != m_status.end(); ++status ){
		const GID &gid = *(GID*)&(status->first);
		mail = mails[gid.mail_id];

		if( mail == mails.end() ){
			DeleteMail(status->first, 
		}
	}

	// ύXɂȂ郁[̓
	for( mail = mails.begin(); mail != mails.end(); ++mail ){
		const CMailInfo &mi = *(mail->second);

		nxt_s = RM_NOT_SET;
		status = m_status.find(mi.gid());

		if( maxMail <= 0 ){
			if( status != m_status.end() )
				DeleteMail(mi.gid(), mi.gids());

			nxt_s = GetNextStatus(mi, recent_limit);
			mnums[nxt_s]++;
			continue;
		}
		maxMail--;

		cur_s = (status == m_status.end()) ? RM_NOT_SET: status->second;
		nxt_s = GetNextStatus(mi, recent_limit);

		_ASSERT( RM_NOT_SET < nxt_s && nxt_s < NUM_OF_NUM );
		mnums[nxt_s]++;
		if( nxt_s != cur_s ){
			if( cur_s != RM_NOT_SET ){
				m_pNsm->ChangeMailStatus(mi.gids(), nxt_s);
				status->second = nxt_s;
			}else{
				AddMail(mi, nxt_s);
			}
		}
	}
}
*/

// 폜ꂽ[̓
void CRiffNsmProx::MailSyncDelete(const NumberMap &mails)
{
	NumberMap::const_iterator mail;
	StatusMap::iterator status;
	char gids[CGID::GIDS_LEN+1];

	for( status = m_status.begin(); status != m_status.end(); ){
		const CGID gid(status->first);
		mail = mails.find(gid.mail_id());

		if( mail == mails.end() ){
			// ɑ݂Ȃ̂ō폜
			gid.ToStr(gids);
			status = DeleteMail(status, gids); 
		}else{
			++status;
		}
	}

}

// ύXɂȂ郁[̓
void CRiffNsmProx::MailSyncAdd(const NumberMap &mails, int maxMail, int mnums[])
{
	NumberMap::const_iterator mail;
	StatusMap::iterator status;
	time_t recent_limit, new_limit;
	MailStatus cur_s, nxt_s;

	time(&recent_limit);
	new_limit = recent_limit - m_set.interval - 4; // ^C}x̂ŁB
	recent_limit -= m_set.show_term;

	for( mail = mails.begin(); mail != mails.end(); ++mail ){
		const CMailInfo &mi = *(mail->second);
		status = m_status.find(mi.gid());

		cur_s = (status == m_status.end()) ? RM_NOT_SET: status->second;
		nxt_s = GetNextStatus(mi, recent_limit, new_limit);

		_ASSERT( RM_NOT_SET < nxt_s && nxt_s < NUM_OF_NUM );
		mnums[nxt_s]++;
		if( maxMail > 0 ){
			if( nxt_s != cur_s ){
				if( cur_s != RM_NOT_SET ){
					ChangeMailStatus(mi, nxt_s);
				}else{
					AddMail(mi, nxt_s);
				}
			}
			maxMail--;
		}else{
			if( cur_s != RM_NOT_SET )
				DeleteMail(mi.gid(), mi.gids());
		}
	}
}

//******************************************************************
// thread
//******************************************************************

void __cdecl CRiffNsmProx::PreviewThread(void* param)
{
	CPreviewDlg *pd = (CPreviewDlg*)param;

	RIFF_GLOBAL.RegistPrevDlg(pd);
	pd->DoModalWithAccelerator(CNsmPlugin::GetModuleInstance(), NULL);
	RIFF_GLOBAL.UnregistPrevDlg(pd);

	delete pd;
}

