#pragma once
#include <string>
#include "RiffNsm.h"
#include "setting_struct.h"
#include "mail_info.h"
#include "util.h"
#include "general_strings.h"

class CRiffNsmProx
{
private:
	typedef std::map<u_int, MailStatus> StatusMap;

private:
	CRiffNsm* m_pNsm;
	const CGID m_gid;
	UISettings m_set;
	std::wstring m_wAcc;

	StatusMap m_status;
	int m_mnums[NUM_OF_NUM];

public:
	CRiffNsmProx(CRiffNsm *pNsm, CGID gid, UISettings &set);
	const CRiffNsmProx& operator =(const CRiffNsmProx&)
	{ _ASSERT(FALSE); }
	~CRiffNsmProx(void);

public:
// Ǝ+Ăяo
//	void MailUpdate(const NumberMap &mails, 
//		const NumberMap &add, const GIDList &del);
	void MailSync(const NumberMap &mails);
	void NewMail(const NumberMap &adds);
//	void DeleteMail(const CMailInfo &mi);
	void ViewMail(const CMailInfo &mi, const std::string &body);
	void Error(CRiffException e) const;

// Ăяo
	void BeginCheck() const
	{ m_pNsm->BeginCheck(); }
	void EndCheck() const
	{ m_pNsm->EndCheck(); }
	void NotifyState(LPCSTR msg) const
	{ m_pNsm->NotifyState(m_gid.chkr_id(), m_wAcc.c_str(), msg); }
	void NotifyDefaultState() const
	{ m_pNsm->NotifyDefaultState(m_gid.chkr_id(), m_wAcc.c_str(), m_mnums); }
	bool ConfirmDelete(const CMailInfo &mi, HWND hParent) const
	{ return m_pNsm->ConfirmDelete(mi, hParent); }

// Ǝ
	void NotifyMsg(LPCSTR msg) const
	{
		NSM_SERVICE.DefaultNotify(msg, NMNI_INFORMATION, m_set.notify_term);
	}
	void ConfigChanged(const UISettings &set)
	{
		m_set = set;
		::SJIStoUnicode(m_set.account.c_str(), m_wAcc);
		m_pNsm->UpdateProx(m_gid.chkr_id(), m_wAcc.c_str(), m_mnums);
	}

private:
//	void AddMails(const NumberMap adds);
//	void DeleteMails(const GIDList dels);
//	void UpdateMails(const NumberMap mails, int maxMail, int mnums[]);
	void MailSyncDelete(const NumberMap &mails);
	void MailSyncAdd(const NumberMap &mails, int maxMail, int mnums[]);

	void CopyMNums(int dst[], const int src[]) const
	{ for( int i = 0; i < NUM_OF_NUM; i++ ) dst[i] = src[i]; }

	void MailStatusUpdated(int old_mnums[], int new_mnums[])
	{
		m_pNsm->MailStatusUpdated(m_gid.chkr_id(), m_wAcc.c_str(), old_mnums, new_mnums);
	}
	void AddMail(const CMailInfo &mi, MailStatus s)
	{
		m_pNsm->AddMail(m_gid.chkr_id(), mi, s);
		m_status[mi.gid()] = s;
	}
	void DeleteMail(u_int gid, LPCSTR gids)
	{
		m_pNsm->DeleteMail(gids);
		m_status.erase(gid);
	}
	StatusMap::iterator DeleteMail(StatusMap::iterator i, LPCSTR gids)
	{
		m_pNsm->DeleteMail(gids);
		return m_status.erase(i);
	}
	void ChangeMailStatus(const CMailInfo &mi, MailStatus s)
	{
		m_pNsm->ChangeMailStatus(mi.gids(), s);
		m_status[mi.gid()] = s;
	}
	MailStatus GetNextStatus(const CMailInfo &mi, time_t recent_limit, time_t new_limit) const
	{
		if( mi.isPreviewed() ) return RM_PREVIEWED;
		if( mi.recv_time() < recent_limit ) return RM_NORMAL;
		if( mi.recv_time() < new_limit ) return RM_RECENT;
		return RM_NEW;
	}
	void NotifyNewMailNum(int num) const
	{
		char buf[256];

		if( m_set.bDoLog ){
			wchar_t wbuf[256];
			std::string ustr;

			_snwprintf_s(wbuf, sizeof(wbuf), L"%s [M %d", m_wAcc.c_str(), num);
			UnicodeToUTF8(wbuf, ustr);
			m_pNsm->DoLog("M", ustr.c_str());
		}
		_snprintf_s(buf, sizeof(buf), "[%s]\r\n[M %d", m_set.account.c_str(), num);
		m_pNsm->Notify(true, buf, m_set.mnum_notify_type,
			m_set.notify_term, CRiffNsm::MailerServiceCaller, (int)m_gid);
	}
	void NotifyMail(const CMailInfo &mi) const
	{
		std::string info;

		info = "[";
		info += m_set.account;
		info += "]";
		info += CRLF_STRING;
		info += mi.subject();
		info += CRLF_STRING;
		info += mi.from();
		m_pNsm->Notify(false, info.c_str(), m_set.mail_notify_type,
			m_set.notify_term, CRiffNsm::ViewServiceCaller, (int)mi.gid());
	}

private:
	static void __cdecl PreviewThread(void* param);
};
