#include "StdAfx.h"
#include ".\riffplugin.h"
#include "RiffNsm.h"
#include "RiffGlobal.h"
#include "service_name.h"
#include "mleak.h"

#define CALL_RIFF_CONNECTION(sel, func) \
	CRiffConnection *prc = m_cons[sel]; \
	if( prc != NULL ) prc->func;

const wchar_t RW_MAIL_PREVIEW[]	= L"[vr[";
const wchar_t RW_MAIL_DELETE[]	= L"[폜";
const wchar_t RW_CHECK_NOW[]	= L"[`FbN";
const wchar_t RW_OPEN_SETTING[]	= L"ݒ";
const wchar_t RW_BEGIN_CHECK[]	= L"`FbNON";
const wchar_t RW_END_CHECK[]	= L"`FbNOFF";
const wchar_t RW_TOGGLE_CHECK[]	= L"`FbN؂ւ";
const wchar_t RW_G_RUN_MUA[]	= L"[\tgN";

const char REM_WSA_VERSION_ALERT[]	= "";
const char REM_CANT_CREATE_CON[]	= "";

CRiffPlugin CRiffPlugin::m_singleton;
CRiffSettings CRiffSettings::m_singleton(THIS_PROTOCOL);

//******************************************************************
// 
//******************************************************************

CRiffPlugin::CRiffPlugin(void)
{
	m_pApiVer		= NSM_API_VERSION;
	m_pModuleName	= PLUGIN_MODULENAME;
	m_pTitle		= "Riff";
	m_pDescription	= "Regnessem Biff. A Mail Check Plugin.";
	m_pAuthor		= "fors";
	m_pCopyright	= "http://cc.ii2.cc/~fors/sysworks/prog/regne/";
	m_pPluginVer	= "0.2.0"
#ifdef _DEBUG
		__TIMESTAMP__
#endif
		;

	WSADATA wsaData;
	int err;

	m_bWSAStartupSuccess = false;
	err = WSAStartup(MAKEWORD(2,0), &wsaData);
	if( (err == 0 ) && (LOBYTE(wsaData.wVersion) == 2)
		&& (HIBYTE(wsaData.wVersion) == 0) )
	{
		m_bWSAStartupSuccess = true;

		SSL_load_error_strings();
		SSL_library_init();
	}
}

CRiffPlugin::~CRiffPlugin(void)
{
	if( m_bWSAStartupSuccess )
	{
		WSACleanup();
		ERR_free_strings();
	}
}

//******************************************************************
// 
//******************************************************************
int CRiffPlugin::OnInitialize()
{
	HNsmEvent e;
	e = NSM_SERVICE.CreateEvent(RE_DOLOG);

	NSM_SERVICE.CreateService(NMS_COMMON_SHOWOPTIONDIALOG , ServiceShowOptionDialog);

	NSM_SERVICE.CreateService(RA_CONNECT, ServiceConConnect);
	NSM_SERVICE.CreateService(RA_DISCONNECT, ServiceConDisconnect);
	NSM_SERVICE.CreateService(RA_CHANGE_USER_STATUS, ServiceConChangeUserStatus);
//	NSM_SERVICE.CreateService(PLUGIN_MODULENAME "/Connection/RemoveMember", ServiceConRemoveMember);
	NSM_SERVICE.CreateService(RA_SESSION_OPEN, ServiceSesOpen);

	NSM_SERVICE.CreateService(RA_MAIL_PREVIEW, ServiceMailPreview);
	NSM_SERVICE.CreateService(RA_MAIL_DELETE, ServiceMailDelete);
	NSM_SERVICE.CreateService(RA_G_OPEN_SETTING, ServiceOpenSetting);
	NSM_SERVICE.CreateService(RA_M_OPEN_SETTING, ServiceOpenSetting);
	NSM_SERVICE.CreateService(RA_CHECK_NOW, ServiceCheckNow);
	NSM_SERVICE.CreateService(RA_BEGIN_CHECK, ServiceBeginCheck);
	NSM_SERVICE.CreateService(RA_END_CHECK, ServiceEndCheck);
	NSM_SERVICE.CreateService(RA_G_RUN_MUA, ServiceGRunMUA);

	NSM_SERVICE.CreateService(RA_CONFIG_UPDATED, ServiceConfigUpdated);
	NSM_SERVICE.CreateService(RA_PREVIEW_MAIL, ServicePreviewMail);
	NSM_SERVICE.CreateService(RA_DELETE_MAIL, ServiceDeleteMail);
	NSM_SERVICE.CreateService(RA_RUN_MUA, ServiceRunMUA);

	RIFF_SETTINGS.Init();
	CRiffNsm::SetLogEventHandle(e);
	return NMP_ALLRIGHT;
}

void CRiffPlugin::OnTerminate()
{
//	NSM_SERVICE.Terminate();
	RIFF_GLOBAL.DeleteAll();
	m_cons.DeleteAll();
}

int CRiffPlugin::OnSystemModulesLoaded()
{
	if( ! m_bWSAStartupSuccess ){
		CRiffNsm::NotifyError(REM_WSA_VERSION_ALERT);
	}
	return NMP_ALLRIGHT;
}


//******************************************************************
// T[rX
//******************************************************************

// ftHgݒ
int CRiffPlugin::DoShowOptionDialog(HWND hWnd)
{
	RIFF_SETTINGS.ShowOptionDialog(GetModuleInstance(), hWnd);
	return 0;
}

//-----------------------------------------------------
// vgRT[rX

// ڑ
int CRiffPlugin::DoConConnect(PLoginInfo pli)
{
	if( ! m_bWSAStartupSuccess ){
		CRiffNsm::NotifyError(REM_WSA_VERSION_ALERT);
		return 0;
	}

	return m_cons.Create(pli);
}

// ؒf
int CRiffPlugin::DoConDisconnect(HNsmConnection hConnection)
{
	m_cons.Delete(hConnection);
	return 0;
}

// `FbNON/OFF
int CRiffPlugin::DoConChangeUserStatus(HNsmConnection hConnection, PUserStatusInfo pusi)
{
	CRiffConnection *prc = m_cons[hConnection];

	if( prc != NULL ){
		switch( pusi->nStatus ){
		case NMST_ONLINE:
			prc->BeginCheck();
			break;
		default:
			prc->EndCheck();
			break;
		}
	}

	return 0;
}

// ZbVI[v = vr[
int CRiffPlugin::DoSesOpen(HNsmConnection hConnection, POpenSessionInfo posi)
{
	CALL_RIFF_CONNECTION(hConnection, ViewMail(posi->lpToAccount));
	return 0;
}

//-----------------------------------------------------
// UIT[rX

// vr[j[
int CRiffPlugin::DoMailPreview(HNsmConnection hConnection, PNsmUIServiceInfo pnuis)
{
	switch( pnuis->nInfoKey ){
	HANDLE_GETCAPTION(pnuis, RW_MAIL_PREVIEW);
	case NMUI_ONCLICK:
		CALL_RIFF_CONNECTION(hConnection, ViewMail(pnuis->lpAccount));
		break;
	}
	return 1;
}

// 폜j[
int CRiffPlugin::DoMailDelete(HNsmConnection hConnection, PNsmUIServiceInfo pnuis)
{
	switch( pnuis->nInfoKey ){
	HANDLE_GETCAPTION(pnuis, RW_MAIL_DELETE);
	case NMUI_ONCLICK:
		CALL_RIFF_CONNECTION(hConnection, DeleteMail(pnuis->lpAccount));
		break;
	}
	return 1;
}

// ʐݒ
int CRiffPlugin::DoOpenSetting(HNsmConnection hConnection, PNsmUIServiceInfo pnuis)
{
	switch( pnuis->nInfoKey ){
	HANDLE_GETCAPTION(pnuis, RW_OPEN_SETTING);
	case NMUI_ONCLICK:
		CALL_RIFF_CONNECTION(hConnection, OpenSetting(hConnection, pnuis->nGroupId));
		break;
	}
	return 1;
}

// 蓮`FbNj[
int CRiffPlugin::DoCheckNow(HNsmConnection hConnection, PNsmUIServiceInfo pnuis)
{
	switch( pnuis->nInfoKey ){
	HANDLE_GETCAPTION(pnuis, RW_CHECK_NOW);
	case NMUI_ONCLICK:
		CALL_RIFF_CONNECTION(hConnection, CheckNow(pnuis->nGroupId));
		break;
	}
	return 1;
}

int CRiffPlugin::DoBeginCheck(HNsmConnection hConnection, PNsmUIServiceInfo pnuis)
{
	switch( pnuis->nInfoKey ){
	HANDLE_GETCAPTION(pnuis, RW_BEGIN_CHECK);
	case NMUI_ONCLICK:
		CALL_RIFF_CONNECTION(hConnection, BeginCheck(pnuis->nGroupId));
		break;
	}
	return 1;
}

int CRiffPlugin::DoEndCheck(HNsmConnection hConnection, PNsmUIServiceInfo pnuis)
{
	switch( pnuis->nInfoKey ){
	HANDLE_GETCAPTION(pnuis, RW_END_CHECK);
	case NMUI_ONCLICK:
		CALL_RIFF_CONNECTION(hConnection, EndCheck(pnuis->nGroupId));
		break;
	}
	return 1;
}

int CRiffPlugin::DoGRunMUA(HNsmConnection hConnection, PNsmUIServiceInfo pnuis)
{
	switch( pnuis->nInfoKey ){
	HANDLE_GETCAPTION(pnuis, RW_G_RUN_MUA);
	case NMUI_ONCLICK:
		CALL_RIFF_CONNECTION(hConnection, RunMUA(pnuis->nGroupId));
		break;
	}
	return 1;
}

//-----------------------------------------------------
// T[rX

// ݒXV
int CRiffPlugin::DoConfigUpdated(HNsmConnection hConnection, int nAcc)
{
	if( hConnection != NULL ){
		CALL_RIFF_CONNECTION(hConnection, ConfigChanged(nAcc));
	}else{
		m_cons.ConfiguUpdated();
	}
	return 0;
}

int CRiffPlugin::DoPreviewMail(UINT gid)
{
	CALL_RIFF_CONNECTION(gid, ViewMail(gid));
	return 0;
}

int CRiffPlugin::DoDeleteMail(UINT gid, HWND hParent)
{
	CALL_RIFF_CONNECTION(gid, DeleteMail(gid, hParent));
	return 0;
}

int CRiffPlugin::DoRunMUA(UINT gid)
{
	CALL_RIFF_CONNECTION(gid, RunMUA(gid));
	return 0;
}

//******************************************************************
// 
//******************************************************************

