#include "StdAfx.h"
#include <sstream>
#include <process.h>
#include <algorithm>
#include ".\riffsettings.h"
#include "RiffGlobal.h"
#include "DefaultsPage.h"
#include "MultiPage.h"
#include "SinglePage.h"
#include "SettingDlg.h"
#include "service_name.h"
#include "DynamicArray.h"
#include "NsmPlugin.h"
#include "CommonService.h"
#include "PSheet.h"
#include "mleak.h"

const static char RS_DEFAULT[]			= "Default";
const static char RS_ACCOUNTS[]			= "Accounts";
const static char RS_ALLACCOUNTS[]		= "AllAccounts";

const static char RS_HOST[]				= "Host";
const static char RS_PORT[]				= "Port";
const static char RS_AUTH[]				= "Auth";
const static char RS_USER[]				= "User";
const static char RS_PASS[]				= "Pass";
const static char RS_PROTOCOL[]			= "Protocol";
const static char RS_CHECKREALTIME[]	= "CheckRealTime";
const static char RS_PREVIEWLINENUM[]	= "PreviewLineNum";
const static char RS_INTERVAL[]			= "Interval";
const static char RS_SHOWTERM[]			= "ShowTerm";
const static char RS_MAXMAIL[]			= "MaxMail";
const static char RS_MAXNOTIFY[]		= "MaxNotify";
const static char RS_NOTIFYEACH[]		= "NotifyEach";
const static char RS_NOTIFYTIME[]		= "NotifyTime";
const static char RS_WINDOWRECT[]		= "WindowRect";
const static char RS_ACCOUNT[]			= "Account";
const static char RS_FONTSIZE[]			= "FontSize";
const static char RS_FONTNAME[]			= "FontName";
const static char RS_FORCEWRAP[]		= "ForceWrap";
const static char RS_COMLOGTYPE[]		= "ComLogType";
const static char RS_MAILNOTIFYTYPE[]	= "MailNotifyType";
const static char RS_MNUMNOTIFYTYPE[]	= "MNumNotifyType";
const static char RS_MUAPATH[]			= "MuaPath";
const static char RS_MUAOPTION[]		= "MuaOption";
const static char RS_MARKTYPE[]			= "MarkType";
const static char RS_TIMEOUT[]			= "Timeout";
const static char RS_AUTOPREVIEW[]		= "AutoPreview";
const static char RS_SECURITY[]			= "Security";
const static char RS_USEASYNC[]			= "UseAsync";
const static char RS_KEEPCON[]			= "KeepCon";
const static char RS_NOTPREVIEW1ST[]	= "NotPreview1st";

const static char UID_EXT[]			= "uid";
const static char SETTING_DLG_NAME[]	= "Riffݒ";
const LPCSTR CRiffSettings::DEFAULT_FONT = "lr SVbN";

//******************************************************************
// 
//******************************************************************

CRiffSettings::CRiffSettings(LPCTSTR pluginName)
: COptionSettings(pluginName)
{
	m_mydir = GetMyDir();
}

CRiffSettings::~CRiffSettings(void)
{
	SavePrevWndRect(m_r);
	m_ini.Write(RS_DEFAULT, RS_FORCEWRAP, m_bForceWrap);
}

//******************************************************************
// p
//******************************************************************

// ftHgǂݍ
void CRiffSettings::Init()
{
	m_default.preview_linenum	= m_ini.ReadInt(RS_DEFAULT, RS_PREVIEWLINENUM, 20);
	m_default.interval			= m_ini.ReadInt(RS_DEFAULT, RS_INTERVAL, 600);
	m_default.max_mail			= m_ini.ReadInt(RS_DEFAULT, RS_MAXMAIL, 30);
	m_default.timeout			= m_ini.ReadInt(RS_DEFAULT, RS_TIMEOUT, 10);

	m_default.show_term			= m_ini.ReadInt(RS_DEFAULT, RS_SHOWTERM, 2 * 60 * 60);
	m_default.notify_each		= (m_ini.ReadInt(RS_DEFAULT, RS_NOTIFYEACH, 0) != 0);
	m_default.notify_term		= m_ini.ReadInt(RS_DEFAULT, RS_NOTIFYTIME, 10);
	m_default.auto_preview		= (m_ini.ReadInt(RS_DEFAULT, RS_AUTOPREVIEW, 0) != 0);

	m_default.comlog_type		= (ComLogType)m_ini.ReadInt(RS_DEFAULT, RS_COMLOGTYPE, RC_NEW);
	m_default.mail_notify_type	= (NotifyIconType)m_ini.ReadInt(
					RS_DEFAULT, RS_MAILNOTIFYTYPE, NMNI_INFORMATION);
	m_default.mnum_notify_type	= (NotifyIconType)m_ini.ReadInt(
					RS_DEFAULT, RS_MNUMNOTIFYTYPE, NMNI_INFORMATION);
	m_default.font_size = m_ini.ReadInt(RS_DEFAULT, RS_FONTSIZE, 10);
	m_ini.ReadString(m_default.font_name, RS_DEFAULT, RS_FONTNAME, DEFAULT_FONT);
	m_ini.ReadString(m_default.mua_path, RS_DEFAULT, RS_MUAPATH, NULL_STRING);
	m_default.max_notify		= m_ini.ReadInt(RS_DEFAULT, RS_MAXNOTIFY, 10);
	m_default.mark_type			= (RE_MARK)m_ini.ReadInt(RS_DEFAULT, RS_MARKTYPE, RW_RECENT);
	m_default.max_auto_preview	= m_ini.ReadInt(RS_DEFAULT, RS_NOTPREVIEW1ST, 10);

	CreateLogFont(m_lf);
	LoadPrevWndRect(m_r);
	m_bForceWrap = (m_ini.ReadInt(RS_DEFAULT, RS_FORCEWRAP, 0) != 0);

	RIFF_GLOBAL.SetMaxNotify(m_default.max_notify);
}

// AJEgZbgǂݍ
void CRiffSettings::GetAccSettings(int nAcc, AccSettings &set)
{
	char buf[INT_STR_LEN];

	_itoa_s(nAcc, buf, 10);

	set.c_set.protocol = (RE_PROTOCOL)m_ini.ReadInt(buf, RS_PROTOCOL, RP_POP3);

	set.c_set.preview_linenum	= GetValueFromMix(buf, RS_PREVIEWLINENUM, m_default.preview_linenum);
	set.c_set.interval			= GetValueFromMix(buf, RS_INTERVAL, m_default.interval);
	set.c_set.max_mail			= GetValueFromMix(buf, RS_MAXMAIL, m_default.max_mail);
	set.c_set.auto_preview		= (GetValueFromMix(buf, RS_AUTOPREVIEW, m_default.auto_preview) != 0);
	set.c_set.max_auto_preview	= m_default.max_auto_preview;
	m_ini.ReadString(set.c_set.mua_option, buf, RS_MUAOPTION, NULL_STRING);

	GetUISettings(nAcc, set.u_set);
	GetServerInfo(nAcc, set.info);
}

// ftHgAJEgZbgǂݍ
void CRiffSettings::GetDefaultSettings(AccSettings &set)
{
	set.info.timeout = m_default.timeout;

	set.c_set.protocol			= RP_ERROR;
	set.c_set.preview_linenum	= m_default.preview_linenum;
	set.c_set.interval			= m_default.interval;
	set.c_set.max_mail			= m_default.max_mail;
	set.c_set.auto_preview		= m_default.auto_preview;
	set.c_set.max_auto_preview	= m_default.max_auto_preview;

	set.u_set.notify_each = m_default.notify_each;
	set.u_set.bDoLog = (m_default.comlog_type == RC_NEW);
	set.u_set.mail_notify_type = m_default.mail_notify_type;
	set.u_set.mnum_notify_type = m_default.mnum_notify_type;
	set.u_set.show_term = m_default.show_term;
	set.u_set.max_mail = m_default.max_mail;
	set.u_set.notify_term = m_default.notify_term;
}

// T[o擾
void CRiffSettings::GetServerInfo(int nAcc, ServerInfo &info)
{
	char buf[INT_STR_LEN];
	std::string pass;

	_itoa_s(nAcc, buf, 10);
	m_ini.ReadString(info.host, buf, RS_HOST, NULL_STRING);
	m_ini.ReadString(info.user, buf, RS_USER, NULL_STRING);
	m_ini.ReadString(pass, buf, RS_PASS, NULL_STRING);
	info.security	= static_cast<RE_SECURITY>(m_ini.ReadInt(buf, RS_SECURITY, RS_NONE));
	info.auth		= m_ini.ReadInt(buf, RS_AUTH, 1);
	info.timeout	= m_default.timeout;
	info.port		= static_cast<u_short>(m_ini.ReadInt(buf, RS_PORT, 0));
	info.use_async	= m_ini.ReadBool(buf, RS_USEASYNC, true);
	info.keep_con   = m_ini.ReadBool(buf, RS_KEEPCON, false);
	info.bDoLog		= (m_default.comlog_type == RC_COM);
	DecodePassword(pass.c_str(), info.pass);
}

// UI擾
void CRiffSettings::GetUISettings(int nAcc, UISettings &set)
{
	char buf[INT_STR_LEN];

	_itoa_s(nAcc, buf, 10);
	m_ini.ReadString(set.account, buf, RS_ACCOUNT, NULL_STRING);
	set.max_mail			= GetValueFromMix(buf, RS_MAXMAIL, m_default.max_mail);
	set.show_term			= GetValueFromMix(buf, RS_SHOWTERM, m_default.show_term);
	set.notify_each			= (GetValueFromMix(buf, RS_NOTIFYEACH, m_default.notify_each) != 0);
	set.notify_term			= GetValueFromMix(buf, RS_NOTIFYTIME, m_default.notify_term);
	set.mail_notify_type	= m_default.mail_notify_type;
	set.interval			= GetValueFromMix(buf, RS_INTERVAL, m_default.interval);
	set.mnum_notify_type	= m_default.mnum_notify_type;
	set.bDoLog				= (m_default.comlog_type == RC_NEW);
}

// AJEgXg擾
void CRiffSettings::GetAccNoList(AccNoList &list, LPCSTR lpAccount)
{
	tstring str;

	if( lpAccount == NULL ) return;

	m_ini.ReadString(str, RS_ACCOUNTS, lpAccount, NULL_STRING);
	if( str == NULL_STRING ){
		m_ini.ReadString(str, RS_DEFAULT, RS_ALLACCOUNTS, NULL_STRING);
	}
	MakeAccNoList(str, list);
}

//******************************************************************
// ݒ葤
//******************************************************************

// ʐݒp擾
void CRiffSettings::GetFullSettings(int nAcc, FullSettings &s)
{
	char buf[INT_STR_LEN];

	_itoa_s(nAcc, buf, 10);
	GetServerInfo(nAcc, s.server);
	m_ini.ReadString(s.account, buf, RS_ACCOUNT, NULL_STRING);
	m_ini.ReadString(s.mua_option, buf, RS_MUAOPTION, NULL_STRING);
	s.protocol = (RE_PROTOCOL)m_ini.ReadInt(buf, RS_PROTOCOL, RP_POP3);
	ReadDefAndValue(buf, RS_INTERVAL, s.interval, m_default.interval);
	ReadDefAndValue(buf, RS_NOTIFYTIME, s.notify_term, m_default.notify_term);
	ReadDefAndValue(buf, RS_NOTIFYEACH, s.notify_each, m_default.notify_each);
	ReadDefAndValue(buf, RS_MAXMAIL, s.max_mail, m_default.max_mail);
	ReadDefAndValue(buf, RS_SHOWTERM, s.show_term, m_default.show_term);
	ReadDefAndValue(buf, RS_PREVIEWLINENUM, s.preview_linenum, m_default.preview_linenum);
	ReadDefAndValue(buf, RS_AUTOPREVIEW, s.auto_preview, m_default.auto_preview);
}

// ʐݒۑ
void CRiffSettings::SetFullSettings(int nAcc, FullSettings &s)
{
	char buf[INT_STR_LEN];

	_itoa_s(nAcc, buf, 10);

	m_ini.Write(buf, RS_PROTOCOL, s.protocol);
	m_ini.Write(buf, RS_ACCOUNT, s.account.c_str());
	m_ini.Write(buf, RS_MUAOPTION, s.mua_option.c_str());
	SetServerInfo(nAcc, s.server);
	WriteDefAndValue(buf, RS_INTERVAL, s.interval);
	WriteDefAndValue(buf, RS_NOTIFYTIME, s.notify_term);
	WriteDefAndValue(buf, RS_NOTIFYEACH, s.notify_each);
	WriteDefAndValue(buf, RS_MAXMAIL, s.max_mail);
	WriteDefAndValue(buf, RS_SHOWTERM, s.show_term);
	WriteDefAndValue(buf, RS_PREVIEWLINENUM, s.preview_linenum);
	WriteDefAndValue(buf, RS_AUTOPREVIEW, s.auto_preview);
}

// T[oۑ
void CRiffSettings::SetServerInfo(int nAcc, ServerInfo &info)
{
	std::string encpass;
	char buf[INT_STR_LEN];

	_itoa_s(nAcc, buf, 10);
	EncodePassword(info.pass.c_str(), encpass);
	m_ini.Write(buf, RS_HOST, info.host.c_str());
	m_ini.Write(buf, RS_PORT, info.port);
	m_ini.Write(buf, RS_AUTH, info.auth);
	m_ini.Write(buf, RS_USER, info.user.c_str());
	m_ini.Write(buf, RS_PASS, encpass.c_str());
	m_ini.Write(buf, RS_SECURITY, info.security);
	m_ini.Write(buf, RS_USEASYNC, info.use_async);
	m_ini.Write(buf, RS_KEEPCON, info.keep_con);
}

// ftHgݒ擾
void CRiffSettings::GetDefaultSettings(DefaultSettings &set)
{
	set = m_default;
}

// ftHgݒۑ
void CRiffSettings::SetDefaultSettings(DefaultSettings &set)
{
	m_default = set;
	CreateLogFont(m_lf);

	m_ini.Write(RS_DEFAULT, RS_PREVIEWLINENUM, m_default.preview_linenum);
	m_ini.Write(RS_DEFAULT, RS_INTERVAL, m_default.interval);
	m_ini.Write(RS_DEFAULT, RS_MAXMAIL, m_default.max_mail);
	m_ini.Write(RS_DEFAULT, RS_SHOWTERM, m_default.show_term);
	m_ini.Write(RS_DEFAULT, RS_NOTIFYEACH, m_default.notify_each);
	m_ini.Write(RS_DEFAULT, RS_NOTIFYTIME, m_default.notify_term);
	m_ini.Write(RS_DEFAULT, RS_TIMEOUT, m_default.timeout);
	m_ini.Write(RS_DEFAULT, RS_AUTOPREVIEW, m_default.auto_preview);

	m_ini.Write(RS_DEFAULT, RS_MAXNOTIFY, m_default.max_notify);
	m_ini.Write(RS_DEFAULT, RS_COMLOGTYPE, m_default.comlog_type);
	m_ini.Write(RS_DEFAULT, RS_MAILNOTIFYTYPE, m_default.mail_notify_type);
	m_ini.Write(RS_DEFAULT, RS_MNUMNOTIFYTYPE, m_default.mnum_notify_type);
	m_ini.Write(RS_DEFAULT, RS_FONTSIZE, m_default.font_size);
	m_ini.Write(RS_DEFAULT, RS_FONTNAME, m_default.font_name.c_str());
	m_ini.Write(RS_DEFAULT, RS_MUAPATH, m_default.mua_path.c_str());
	m_ini.Write(RS_DEFAULT, RS_MARKTYPE, m_default.mark_type);
	m_ini.Write(RS_DEFAULT, RS_NOTPREVIEW1ST, m_default.max_auto_preview);

	RIFF_GLOBAL.SetMaxNotify(set.max_notify);
	RIFF_GLOBAL.FontUpdated(m_lf);
}

void CRiffSettings::GetAllAccountInfo(AccountList &list, AccountMap &map)
{
	CIniFile::EntryList accounts;
	CIniFile::EntryList::iterator i;
	AccNoList *noList;

	// VOAJEg̃[h
	LoadAccountList(list, map);

	// mapɃ}`AJEgo^
	m_ini.ReadSection(RS_ACCOUNTS, accounts);
	for( i = accounts.begin(); i != accounts.end(); ++i ){
		const std::string &acc = i->first;
		std::string &naccs = i->second;

		if( map.find(acc) == map.end() ){
			noList = new AccNoList;
			MakeAccNoList(naccs, *noList);
			if( noList->size() > 0 ){
				map[acc] = noList;
			}else{
				delete noList;
			}
		}
	}
}

void CRiffSettings::SetAllAccountInfo(AccountList &list, AccountMap &map)
{
	AccountList::iterator li;
	AccountMap::iterator mi;
	AccNoList::iterator ni;
	AccNoMap nomap;
	AccNoMap::iterator i;
	std::ostringstream all;

	// AJEgNökpXg
	MakeAccNoMap(list, nomap);

	// ̃ZNVj
	m_ini.DeleteSection(RS_ACCOUNTS);
	DeleteAllAccountSections();

	// o
	for( li = list.begin(); li != list.end(); ++li ){
		_ASSERT( nomap.find(li->first) != nomap.end() );
		SetFullSettings(nomap[li->first], *(li->second));
		all << li->first << SPACE_DELIMITER;
	}
	m_ini.Write(RS_DEFAULT, RS_ALLACCOUNTS, all.str().c_str());

	for( mi = map.begin(); mi != map.end(); ++mi ){
		const std::string &acc = mi->first;
		AccNoList &naccs = *(mi->second);
		std::ostringstream ss;

		for( ni = naccs.begin(); ni != naccs.end(); ++ni ){
			i = nomap.find(*ni);
			if( i != nomap.end() )
				ss << i->second << SPACE_DELIMITER;
		}
		m_ini.Write(RS_ACCOUNTS, acc.c_str(), ss.str().c_str());
	}
}

void CRiffSettings::UnloadAllAccountInfo(AccountList &list, AccountMap &map)
{
	AccountList::iterator li;
	AccountMap::iterator mi;

	for( li = list.begin(); li != list.end(); ++li )
		delete li->second;
	for( mi = map.begin(); mi != map.end(); ++mi )
		delete mi->second;
}

//******************************************************************
// ݒ_CAO
//******************************************************************

// S̐ݒ_CAO
void CRiffSettings::ShowOptionDialog(HINSTANCE hInst, HWND hParent)
{
	DefaultSettings def;
	AccountList al;
	AccountMap am;

	GetDefaultSettings(def);
	GetAllAccountInfo(al, am);

	CDefaultsPage dp(def);
	CSinglePage sp(al, am, def);
	CMultiPage mp(al, am);
	CPSheet ps;

	ps.Add(dp);
	ps.Add(sp);
	ps.Add(mp);

	if( ps.DoModal(hInst, hParent, SETTING_DLG_NAME) ){
		SetDefaultSettings(def);
		SetAllAccountInfo(al, am);
		NSM_SERVICE.CallService(RA_CONFIG_UPDATED, 0, 0);
	}
	UnloadAllAccountInfo(al, am);

}

// ʐݒ_CAO
void CRiffSettings::ShowAccOptionDialog(HNsmConnection hConnection, int nAcc)
{
	_ASSERT( nAcc >= 0 );
	ThreadParam *p = new ThreadParam;
	p->hConnection = hConnection;
	p->nAcc = nAcc;
	_beginthread(SettingThread, 0, p);
}

//******************************************************************
// 
//******************************************************************

void CRiffSettings::EncodePassword(LPCSTR plain, std::string &enc)
{
	bool pre, value;
	int cnt = 0;

	enc = NULL_STRING;
	srand(GetTickCount());

	pre = ((*plain & 1) != 0);
	if( pre != false ) enc += D2A(0);
	for( LPCSTR pos = plain; *pos; pos++ ){
		for( char i = 0; i < 8; i++ ){
			value = ((*pos & (1 << i)) != 0);
			if( pre == value ){
				if( cnt >= 9 ){
					enc += D2A(9);
					cnt = 1;
				}else{
					cnt++;
				}
			}else{
				enc += D2A(static_cast<char>(cnt));
				pre = value;
				cnt = 1;
			}
		}
	}
	enc += D2A(static_cast<char>(cnt));
}

void CRiffSettings::DecodePassword(LPCSTR enc, std::string &plain)
{
	char c = 0;
	char cnt = 0;
	char value;
	bool b = false;

	plain = NULL_STRING;
	for( LPCSTR pos = enc; *pos; pos++ ){
		value = (*pos & 0xF);
		for( char i = 0; i < value; i++ ){
			c |= (b << cnt++);
			if( cnt > 7 ){
				plain += c;
				cnt = 0;
				c = 0;
			}
		}
		if( value != 9 ) b = !b;
	}
}

void CRiffSettings::LoadDefaultRect(RECT &r)
{
	int screen_x = GetSystemMetrics(SM_CXSCREEN);
	int screen_y = GetSystemMetrics(SM_CYSCREEN);
	int width = 400;
	int height = 300;

	r.left = (screen_x - width)/2;
	r.top = (screen_y - height)/2;
	r.right = r.left + width;
	r.bottom = r.top + height;
}

void CRiffSettings::LoadPrevWndRect(RECT &r)
{
	tstring str;
	m_ini.ReadString(str, RS_DEFAULT, RS_WINDOWRECT, NULL_STRING);

	if( str.length() < 7 ){
		LoadDefaultRect(r);
	}else{
		std::istringstream ss(str.c_str(), str.length());
		ss >> r.left;
		ss >> r.top;
		ss >> r.right;
		ss >> r.bottom;
	}
}

void CRiffSettings::SavePrevWndRect(RECT &r)
{
	std::ostringstream rect;

	rect << r.left << SPACE_DELIMITER;
	rect << r.top << SPACE_DELIMITER;
	rect << r.right << SPACE_DELIMITER;
	rect << r.bottom;

	m_ini.Write(RS_DEFAULT, RS_WINDOWRECT, rect.str().c_str());
}

void CRiffSettings::CreateLogFont(LOGFONT &lf)
{
	memset(&lf, 0, sizeof(lf));
	lf.lfWeight = FW_NORMAL;
	lf.lfCharSet = DEFAULT_CHARSET;
	lf.lfHeight = 12;
	lf.lfWidth = 0;

	lf.lfHeight = m_default.font_size;
	strncpy_s(lf.lfFaceName, m_default.font_name.c_str(), LF_FACESIZE);
}

void CRiffSettings::MakeAccNoMap(AccountList &list, AccNoMap &map)
{
	if( list.size() <= 0 ) return;
	DynamicArray<int> nos(list.size());
	AccountList::iterator li;
	int count = 0;

	for( li = list.begin(); li != list.end(); ++li ){
		nos[count++] = li->first;
	}
	std::sort(&nos[0], &nos[0] + count);

	while( --count >= 0 ){
		map[nos[count]] = count;
	}
}

void CRiffSettings::MakeAccNoList(const tstring &naccs, AccNoList &list)
{
	std::istringstream ss(naccs.c_str(), naccs.size());
	int nAcc;

	for(;;){
		ss >> nAcc;
		if( ss.fail() ) break;
		list.push_back(nAcc);
	}
}

void CRiffSettings::DeleteAllAccountSections()
{
	CIniFile::SectionList sections;
	CIniFile::SectionList::iterator si;

	m_ini.GetSectionList(sections);
	for( si = sections.begin(); si != sections.end(); ++si ){
		if( (atoi(si->c_str()) != 0) || (*si == "0") ){
			m_ini.DeleteSection(si->c_str());
		}
	}
}

// VOAJEg̃AJEglistɂ݂݁A
// mapɃVOAJEgo^
void CRiffSettings::LoadAccountList(AccountList &list, AccountMap &map)
{
	CIniFile::SectionList sections;
	CIniFile::SectionList::iterator si;
	AccNoList *noList;
	FullSettings *pset;
	int nAcc;

	m_ini.GetSectionList(sections);
	for( si = sections.begin(); si != sections.end(); ++si ){
		if( (atoi(si->c_str()) != 0) || (*si == "0") ){
			tstring acc;
			m_ini.ReadString(acc, si->c_str(), RS_ACCOUNT, NULL_STRING);
			nAcc = atoi(si->c_str());

			noList = new AccNoList;
			noList->push_back(ACCNO_SINGLE);
			noList->push_back(nAcc);
			map[acc] = noList;

			pset = new FullSettings;
			GetFullSettings(nAcc, *pset);
			list[nAcc] = pset;
		}
	}
}

//******************************************************************
// 
//******************************************************************

void CRiffSettings::SettingThreadBody(ThreadParam &p)
{
	FullSettings set;
	DefaultSettings def;

	GetFullSettings(p.nAcc, set);
	GetDefaultSettings(def);

	CSettingDlg dlg(set, def);

	if( dlg.DoModal(CNsmPlugin::GetModuleInstance(), NULL) == IDOK ){
		SetFullSettings(p.nAcc, set);
		NSM_SERVICE.CallService(RA_CONFIG_UPDATED, p.hConnection, p.nAcc);
	}
}

void __cdecl CRiffSettings::SettingThread(void* param)
{
	ThreadParam *pParam = (ThreadParam*)param;
	RIFF_SETTINGS.SettingThreadBody(*pParam);
	delete pParam;
}
