#pragma once
#include <string>
#pragma warning( disable: 4702 )
	#include <vector>
	#include <map>
#pragma warning( default: 4702 )
#include "setting_struct.h"
#include "optionsettings.h"
#include "general_strings.h"

//******************************************************************

#define RIFF_SETTINGS	(CRiffSettings::GetInstance())

//******************************************************************

//******************************************************************

class CRiffSettings :
	public COptionSettings
{
// singleton -->
private:
	static CRiffSettings m_singleton;

private:
	CRiffSettings(LPCTSTR pluginName);
	~CRiffSettings(void);

public:
	static CRiffSettings& GetInstance()
	{
		return m_singleton;
	}
// <-- singleton

private:
	typedef std::map<int, int> AccNoMap;
	struct ThreadParam
	{
		HNsmConnection hConnection;
		int nAcc;
	};

private:
	enum{ INT_STR_LEN=16 };
	const static LPCSTR DEFAULT_FONT;

public:
	void Init();

	void GetAccSettings(int nAcc, AccSettings &set);
	void GetDefaultSettings(AccSettings &set);
	void GetServerInfo(int nAcc, ServerInfo &info);
	void GetUISettings(int nAcc, UISettings &set);
	void GetAccNoList(AccNoList &list, LPCSTR lpAccount);

	void GetFullSettings(int nAcc, FullSettings &s);
	void SetFullSettings(int nAcc, FullSettings &s);
	void SetServerInfo(int nAcc, ServerInfo &info);
	void GetDefaultSettings(DefaultSettings &set);
	void SetDefaultSettings(DefaultSettings &set);

	void ShowOptionDialog(HINSTANCE hInst, HWND hParent);
	void ShowAccOptionDialog(HNsmConnection hConnection, int nAcc);

	void GetAllAccountInfo(AccountList &list, AccountMap &map);
	void SetAllAccountInfo(AccountList &list, AccountMap &map);
	void UnloadAllAccountInfo(AccountList &list, AccountMap &map);

	void LoadPrevWndInfo(RECT &r, LOGFONT &lf, bool &bFroceWrap)
	{
		r = m_r;
		lf = m_lf;
		bFroceWrap = m_bForceWrap;
	}
	void SavePrevWndInfo(RECT &r, bool bForceWrap)
	{
		m_r = r;
		m_bForceWrap = bForceWrap;
	}
	LPCSTR GetMuaPath() const
	{ return m_default.mua_path.c_str(); }
	int GetMarkType() const
	{ return m_default.mark_type; }

private:
	void ReadDefAndValue(LPCSTR acc, LPCSTR key, DefAndValue &value, int defaultValue)
	{
		tstring str;
		m_ini.ReadString(str, acc, key, NULL_STRING);
		if( str.length() < 3 ){
			value.use_default = true;
			value.value = defaultValue;
		}else{
			value.use_default = (str[0] != '0');
			value.value  = atoi(str.c_str() + 2);
		}
	}
	void WriteDefAndValue(LPCSTR acc, LPCSTR key, DefAndValue &value)
	{
		char buf[32];

		_snprintf_s(buf, sizeof(buf), "%d %d", value.use_default, value.value );
		m_ini.Write(acc, key, buf);
	}
	int GetValueFromMix(LPCSTR acc, LPCSTR key, int defaultValue)
	{
		DefAndValue v;
		ReadDefAndValue(acc, key, v, defaultValue);
		return v.use_default ? defaultValue: v.value;
	}
	char D2A(char n)
	{
		return static_cast<char>((((rand()*5)/(RAND_MAX+1)) * 0x10 + '0') + n);
	}
	void EncodePassword(LPCSTR plain, std::string &enc);
	void DecodePassword(LPCSTR enc, std::string &plain);

	void LoadDefaultRect(RECT &r);
	void LoadPrevWndRect(RECT &r);
	void SavePrevWndRect(RECT &r);
	void CreateLogFont(LOGFONT &lf);

	void MakeAccNoMap(AccountList &list, AccNoMap &map);
	void MakeAccNoList(const tstring &naccs, AccNoList &list); 
	void DeleteAllAccountSections();
	void LoadAccountList(AccountList &list, AccountMap &map);

	void SettingThreadBody(ThreadParam &p);
	static void __cdecl SettingThread(void* param);

private:
	std::string m_mydir;
	DefaultSettings m_default;
	LOGFONT m_lf;
	RECT m_r;
	bool m_bForceWrap;
};

