#include "stdafx.h"
#include "codec.h"
#include "DynamicArray.h"
#include "general_strings.h"
#include "util.h"
#include "cs_conv.h"

const static char MIME_CHARSET_JIS[]	= "iso-2022-jp";
const static char MIME_CHARSET_UTF7[]	= "utf-7";
const static char MIME_CHARSET_UTF8[]	= "utf-8";

// [wb_fR[h
void Decode(std::string &in, std::string &out)
{
	DynamicArray<char> buf(in.length()+1);
	LPCSTR st = in.c_str();
	LPCSTR ed = st + in.length();
	LPSTR bpos = buf;
	LPSTR bed;
	LPCSTR pos;

/*
	abcdef =?abcdef?= abcde
	|--A--|-----B----|--C---
	st    pos
*/
	pos = st;
	while( st < ed ){
		// ̈A
		pos = strstr(st, "=?");
		if( pos == NULL ) break;

		// 󔒗Ȃ΂
		if( ! IsWSP(st, pos) ){
			if( IsJIS(st, pos) ) bed = JIStoSJIS(bpos, st, pos);
			else bed = Unquote(bpos, st, pos);

			// ^uXy[X
			for( ; bpos < bed; bpos++ )
				if( *bpos == '\t' ) *bpos = ' ';
		}

		st = pos;

		// ̈B
		if( (pos = strchr(pos + 2, '?')) != NULL ){
			if( (pos = strchr(pos + 1, '?')) != NULL ){
				if( (pos = strstr(pos + 1, "?=")) != NULL ){
					st += 2;
					bpos = DecodeMime(bpos, st, pos);
					st = pos + 2;
					while( (0 < *st) && (*st <= ' ') ) st++;
					continue;
				}
			}
		}
		st++;
	}
	if( IsJIS(st, ed) ) JIStoSJIS(bpos, st, ed);
	else Unquote(bpos, st, ed);

	out = buf;
}

// st  ed ܂ł̕񂪋󔒗񂩂ǂf
bool IsWSP(LPCSTR st, LPCSTR ed)
{
	for( ; st < ed; st++ ){
		if( (*st == '\0') || (' ' < *st) )
			return false;
	}

	return true;
}

// in_st  in_ed ܂ł̃NI[gʏ핶ɕϊ
// out ̃obt@Ɋi[B
// out ɂ͏\ȃobt@̂ƂB
// ϊ㕶̍Ō̃kւ̃|C^Ԃ
LPSTR Unquote(LPSTR out, LPCSTR in_st, LPCSTR in_ed)
{
	bool bQuoted = false;
	bool bEscape = false;

	for( LPCSTR pos = in_st; pos < in_ed; pos++ ){
		switch( *pos ){
		case '"':
			if( ! bEscape ){
				bQuoted = ! bQuoted;
				continue;
			}
			break;
		case '\\':
			if( ! bEscape ){
				bEscape = true;
				continue;
			}
			break;
		default:
			break;
		}
		*out++ = *pos;
		bEscape = false;
	}

	*out = '\0';
	return out;
}

// in_st  in_ed ܂łMIMESJISɕϊ
// out ̃obt@Ɋi[B
// out ɂ͏\ȃobt@̂ƂB
// ϊ㕶̍Ō̃kւ̃|C^Ԃ
LPSTR DecodeMime(LPSTR out, LPCSTR in_st, LPCSTR in_ed)
{
	DynamicArray<char> bdec(in_ed-in_st+1);
	LPCSTR pos;
	LPCSTR buf_st, buf_ed;
	LPSTR ret;
	char e;

/*
 =?ISO-2022-JP?B?abcdefgh?=
   |          |          |
   in_st      pos        in_ed
*/
	pos = strchr(in_st, '?');

	e = static_cast<char>(toupper(*(pos+1)));
	pos += 3;

	// oCifR[h
	if( e == 'B' ){
		buf_st = bdec;
		buf_ed = DecodeBase64(bdec, pos, in_ed);
	}else if( e == 'Q' ){
		buf_st = bdec;
		buf_ed = DecodeQuoted(bdec, pos, in_ed);
	}else{
		buf_st = in_st;
		buf_ed = in_ed;
	}

	// eLXgfR[h
	if( _strnicmp(in_st, MIME_CHARSET_JIS, LEN_OF(MIME_CHARSET_JIS)) == 0 ){
		ret = JIStoSJIS(out, buf_st, buf_ed);
	}else if( _strnicmp(in_st, MIME_CHARSET_UTF7, LEN_OF(MIME_CHARSET_UTF7)) == 0 ){
		std::wstring wstr;
		std::string str;

		UTF7toUnicode(buf_st, wstr);
		UnicodeToSJIS(wstr.c_str(), str);
		strcpy_s(out, str.length(), str.c_str());
		ret = out + str.length();
	}else if( _strnicmp(in_st, MIME_CHARSET_UTF8, LEN_OF(MIME_CHARSET_UTF8)) == 0 ){
		std::wstring wstr;
		std::string str;

		UTF8toUnicode(buf_st, wstr);
		UnicodeToSJIS(wstr.c_str(), str);
		strcpy_s(out, str.length(), str.c_str());
		ret = out + str.length();
	}else{
		memcpy(out, buf_st, buf_ed - buf_st);
		ret = out + (buf_ed - buf_st);
	}

	return ret;
}

// in_st  in_ed ܂łURLGXP[voCiɕϊ
// out ̃obt@Ɋi[B
// out ɂ͏\ȃobt@̂ƂB
// ϊ㕶̍Ō̃kւ̃|C^Ԃ
LPSTR DecodeURL(LPSTR out, LPCSTR in_st, LPCSTR in_ed)
{
	for( LPCSTR pos = in_st; pos < in_ed; pos++ ){
		if( *pos == '%' )
		// URLGXP[v
		{
			if( in_ed <= pos + 2 ){
				// Ȃ
				_ASSERT(FALSE);
				break;
			}
			*out = (DecodeQHexToBin(*++pos) << 4);
			*out |= DecodeQHexToBin(*++pos);
			out++;
		}
		else
		// ʏ핶
		{
			*out++ = *pos;
		}
	}

	*out = '\0';
	return out;
}

// in_st  in_ed ܂łquoted printableoCiɕϊ
// out ̃obt@Ɋi[B
// out ɂ͏\ȃobt@̂ƂB
// ϊ㕶̍Ō̃kւ̃|C^Ԃ
LPSTR DecodeQuoted(LPSTR out, LPCSTR in_st, LPCSTR in_ed)
{
	for( LPCSTR pos = in_st; pos < in_ed; pos++ ){
		if( *pos == '=' )
		// quoted printable
		{
			if( pos + 2 >= in_ed ){
				// Ȃ
				_ASSERT(FALSE);
				break;
			}

			if( *(pos+1) == '\r' )
			// s̃XLbv
			{
				pos += 2;
			}
			else
			// 16iGR[h
			{
				*out = DecodeQHexToBin(*++pos) << 4;
				*out |= DecodeQHexToBin(*++pos);
				out++;
			}
		}
		else
		// ʏ핶
		{
			*out++ = *pos;
		}
	}

	*out = '\0';
	return out;
}

// in_st  in_ed ܂łbase64oCiɕϊ
// out ̃obt@Ɋi[B
// out ɂ͏\ȃobt@̂ƂB
// ϊ㕶̍Ō̃kւ̃|C^Ԃ
LPSTR DecodeBase64(LPSTR out, LPCSTR in_st, LPCSTR in_ed)
{
	char c;
	int shift = 3 * 8;
	u_int value = 0;

	for( LPCSTR pos = in_st; pos < in_ed; pos++ ){
		c = DecodeBase64AtoB(*pos);
		if( c < -1 ) continue;

		if( c == -1 )
		// '=' ꍇ
		{
			*out++ = static_cast<char>((value & 0x00FF0000) >> 16);
//			if( flag == 2 ) break;
			if( shift == 12 ) break;
			*out++ = static_cast<char>((value & 0x0000FF00) >> 8);
			break;
		}
		else
		// base64
/*		{
			switch( flag ){
			case 0: value = (c << 18); break;
			case 1: value |= (c << 12); break;
			case 2: value |= (c << 6); break;
			case 3: value |= c;
				*out++ = (value & 0xFF0000) >> 16;
				*out++ = (value & 0xFF00) >> 8;
				*out++ = (value & 0xFF);
				flag = 0;
				continue;
			}
		}
*/		{
			shift -= 6;
			value |= (c << shift);

			if( shift <= 0 ){
				*out++ = static_cast<char>((value & (0xFF << 16)) >> 16);
				*out++ = static_cast<char>((value & (0xFF << 8)) >> 8);
				*out++ = static_cast<char>((value & (0xFF << 0)) >> 0);

				value = 0;
				shift = 24;
			}
		}
	}

	*out = '\0';
	return out;
}

// in_st  in_ed ܂ł̃oCibase64ɕϊ
// out ̃obt@Ɋi[B
// out ɂ͏\ȃobt@̂ƂB
// ϊ㕶̍Ō̃kւ̃|C^Ԃ
LPSTR EncodeBase64(LPCSTR in, LPSTR out, int len)
{
	u_char* end = (u_char*)in + len - 2;
	u_char* pos;
	int value;

	// 3 byteϊ
	for( pos = (u_char*)in; pos < end; ){
		value = *pos++ << 16;
		value |= *pos++ << 8;
		value |= *pos++;
		*out++ = EncodeBase64BtoA(static_cast<u_char>((value & 0xFC0000) >> 18));
		*out++ = EncodeBase64BtoA(static_cast<u_char>((value & 0x3F000) >> 12));
		*out++ = EncodeBase64BtoA(static_cast<u_char>((value & 0xFC0) >> 6));
		*out++ = EncodeBase64BtoA(static_cast<u_char>((value & 0x3F) >> 0));
	}

	if( pos == end )
	// 2 byte]ꍇ
	{
		value = *pos++ << 16;
		value |= *pos++ << 8;
		*out++ = EncodeBase64BtoA(static_cast<u_char>((value & 0xFC0000) >> 18));
		*out++ = EncodeBase64BtoA(static_cast<u_char>((value & 0x3F000) >> 12));
		*out++ = EncodeBase64BtoA(static_cast<u_char>((value & 0xFC0) >> 6));
		*out++ = 'A';
	}
	else if( pos - 1 == end )
	// 1 byte]ꍇ
	{
		value = *pos++ << 16;
		*out++ = EncodeBase64BtoA(static_cast<u_char>((value & 0xFC0000) >> 18));
		*out++ = EncodeBase64BtoA(static_cast<u_char>((value & 0x3F000) >> 12));
		*out++ = 'A';
		*out++ = 'A';
	}

	*out = '\0';
	return out;
}

// 16iΉ鐔lɕϊ
inline char DecodeQHexToBin(char c)
{
	if( ('0' <= c) && (c <= '9') ){ return c - '0'; }
	else if( ('A' <= c) && (c <= 'F') ){ return c - 'A' + 10; }
	else{ _ASSERT(FALSE); return 0; }
}

// base64
//  c ɑΉ4bitoCiԂ
// c == '=' Ȃ -1 
// ȏȊO -2 
inline char DecodeBase64AtoB(char c)
{
	static char atob[] = {
		-2,-2,-2,-2,-2,-2,-2,-2,-2,-2,-2,-2,-2,-2,-2,-2,
		-2,-2,-2,-2,-2,-2,-2,-2,-2,-2,-2,-2,-2,-2,-2,-2,
		-2,-2,-2,-2,-2,-2,-2,-2,-2,-2,-2,62,-2,-2,-2,63,
		52,53,54,55,56,57,58,59,60,61,-2,-2,-2,-1,-2,-2,
		-2, 0, 1, 2, 3, 4, 5, 6, 7, 8, 9,10,11,12,13,14,
		15,16,17,18,19,20,21,22,23,24,25,-2,-2,-2,-2,-2,
		-2,26,27,28,29,30,31,32,33,34,35,36,37,38,39,40,
		41,42,43,44,45,46,47,48,49,50,51,-2,-2,-2,-2,-2,
	};
	char ret;
	if( c & 0x80 ){ _ASSERT(FALSE); return -1; }
	ret = atob[c];
	return ret;
}

// base64
// 4bitoCi b ɑΉ镶Ԃ
inline char EncodeBase64BtoA(unsigned char b)
{
	static char btoa[] = {
		'A','B','C','D','E','F','G','H','I','J','K','L','M','N','O','P',
		'Q','R','S','T','U','V','W','X','Y','Z','a','b','c','d','e','f',
		'g','h','i','j','k','l','m','n','o','p','q','r','s','t','u','v',
		'w','x','y','z','0','1','2','3','4','5','6','7','8','9','+','/'
	};

	_ASSERT( b < 0x40 );
	return btoa[b];
}
